##
# Copyright 2009-2023 The Cyprus Institute
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for BamTools, implemented as an easyblock

@author: Andreas Panteli (The Cyprus Institute)
@author: Kenneth Hoste (Ghent University)
"""
from easybuild.tools import LooseVersion
from easybuild.easyblocks.generic.cmakemake import CMakeMake
from easybuild.easyblocks.generic.makecp import MakeCp
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.systemtools import get_shared_lib_ext


class EB_BamTools(MakeCp, CMakeMake):
    """Support for building and installing BamTools."""

    @staticmethod
    def extra_options(extra_vars=None):
        """Extra easyconfig parameters for BamTools."""
        extra_vars = MakeCp.extra_options()
        extra_vars.update(CMakeMake.extra_options())

        # files_to_copy is not mandatory here, since we overwrite it in install_step
        extra_vars['files_to_copy'][2] = CUSTOM
        # BamTools requires an out of source build
        extra_vars['separate_build_dir'][0] = True

        return extra_vars

    def configure_step(self):
        """Configure BamTools build."""
        CMakeMake.configure_step(self)

    def install_step(self):
        """Custom installation procedure for BamTools."""
        if LooseVersion(self.version) < LooseVersion('2.5.0'):
            self.cfg['files_to_copy'] = ['bin', 'lib', 'include', 'docs', 'LICENSE', 'README']
            MakeCp.install_step(self)
        else:
            CMakeMake.install_step(self)

    def sanity_check_step(self):
        """Custom sanity check for BamTools."""

        shlib_ext = get_shared_lib_ext()

        custom_paths = {
            'files': ['bin/bamtools'],
            'dirs': [],
        }
        if LooseVersion(self.version) < LooseVersion('2.3.0'):
            # bamtools-utils & jsoncpp libs now built as static libs by default since v2.3.0
            custom_paths['files'].extend(['lib/libbamtools-utils.%s' % shlib_ext, 'lib/libjsoncpp.%s' % shlib_ext])
        elif LooseVersion(self.version) < LooseVersion('2.5.0'):
            custom_paths['files'].extend(['include/shared/bamtools_global.h', 'lib/libbamtools.a',
                                          'lib/libbamtools.%s' % shlib_ext, 'lib/libbamtools-utils.a',
                                          'lib/libjsoncpp.a'])
            custom_paths['dirs'].extend(['include/api', 'docs'])
        else:
            custom_paths['files'].extend(['include/bamtools/shared/bamtools_global.h',
                                          ('lib/libbamtools.a', 'lib64/libbamtools.a')])
            custom_paths['dirs'].extend(['include/bamtools/api', ('lib/pkgconfig', 'lib64/pkgconfig')])

        super(EB_BamTools, self).sanity_check_step(custom_paths=custom_paths)



##
# Copyright 2020-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing CRISPR-DAV, implemented as an easyblock.

@author: Denis Kristak (INUITS)
@author: Kenneth Hoste (HPC-UGent)
"""
import glob
import os
from easybuild.easyblocks.generic.binary import Binary
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.filetools import apply_regex_substitutions, back_up_file, write_file
from easybuild.tools.modules import get_software_root


class EB_CRISPR_minus_DAV(Binary):
    """
    Support for building/installing CRISPR-DAV.
    """

    def __init__(self, *args, **kwargs):
        """Constructor for CRISPR-DAV easyblock."""
        super(EB_CRISPR_minus_DAV, self).__init__(*args, **kwargs)
        self.cfg['extract_sources'] = True

    def post_install_step(self):
        """Update configuration files with correct paths to dependencies and files in installation."""

        # getting paths of deps + files we will work with
        example_dir = os.path.join(self.installdir, 'Examples')
        config_file = os.path.join(self.installdir, 'conf.txt')

        dep_roots = {}
        for dep in ('ABRA2', 'BEDTools', 'FLASH', 'Java', 'PRINSEQ', 'pysamstats', 'R', 'SAMtools'):
            root = get_software_root(dep)
            if root:
                dep_roots[dep] = root
            else:
                raise EasyBuildError("Failed to find root directory for %s. Is it included as dependency?", dep)

        # we will be changing both Examples/... conf file (for sanity checks) as well as root (for proper functioning)
        cfg_files = [config_file] + glob.glob(os.path.join(example_dir, 'example*', 'conf.txt'))

        # create a backup of original Examples/example1/conf.txt file + the root conf.txt file
        for filename in cfg_files:
            back_up_file(filename)

        # according to docs, we have to setup conf.txt so that it contains correct paths to dependencies
        # https://github.com/pinetree1/crispr-dav/blob/master/Install-and-Run.md
        # User then has to change conf.txt to include paths to genomes
        # changing both example conf.txt as well as the main one (in root directory) to make it easier for user.

        # func to replace everything in both conf.txt files
        self.modify_conf_files(dep_roots, cfg_files)

        # generating fastq file with correct paths (used for sanity checks)
        for example in ('example1', 'example2'):
            example_dir = os.path.join(example_dir, example)
            fastq_list_file = os.path.join(example_dir, 'fastq.list')
            fastq_list = []
            rawfastq_dir = os.path.join(example_dir, 'rawfastq')

            for x in range(1, 5):
                # we have to use \t or it wont work!
                line = '\t'.join([
                    'sample%s' % x,
                    os.path.join(rawfastq_dir, 'sample%s_R1.fastq.gz' % x),
                    os.path.join(rawfastq_dir, 'sample%s_R2.fastq.gz' % x),
                ])
                fastq_list.append(line)

            # last line should not end with newline (\n)!
            write_file(fastq_list_file, '\n'.join(fastq_list))

    def sanity_check_step(self):
        """Custom sanity check paths for CRISPR-DAV"""
        custom_paths = {
            'files': ['crispr.pl'],
            'dirs': ['Modules', 'Examples', 'Rscripts'],
        }

        # example command from docs - https://github.com/pinetree1/crispr-dav/blob/master/Install-and-Run.md
        # this command is an improvement (no hardcoded stuff) of
        # https://github.com/pinetree1/crispr-dav/blob/master/Examples/example1/run.sh
        example_dir = os.path.join(self.installdir, 'Examples', 'example1')
        outfile = os.path.join(self.builddir, 'test.out')
        example_cmd = ' '.join([
            os.path.join(self.installdir, 'crispr.pl'),
            "--conf %s" % os.path.join(example_dir, 'conf.txt'),
            "--region %s" % os.path.join(example_dir, 'amplicon.bed'),
            "--crispr %s" % os.path.join(example_dir, 'site.bed'),
            "--sitemap %s" % os.path.join(example_dir, 'sample.site'),
            "--fastqmap %s" % os.path.join(example_dir, 'fastq.list'),
            "--conf %s" % os.path.join(example_dir, 'conf.txt'),
            "--genome genomex",
            "2>&1 | tee %s" % outfile,
            " && grep 'Generated HTML report for GENEX_CR1' %s" % outfile,
        ])

        custom_commands = [
            "crispr.pl --help 2>&1 | grep 'Usage: '",
            example_cmd,
        ]

        super(EB_CRISPR_minus_DAV, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)

    def make_module_extra(self):
        txt = super(EB_CRISPR_minus_DAV, self).make_module_extra()
        txt += self.module_generator.prepend_paths('PATH', [''])
        return txt

    def modify_conf_files(self, dep_roots, cfg_files):
        """Replace hardcoded paths in config files."""

        abra2_jar = os.path.join(dep_roots['ABRA2'], 'abra2-%s.jar' % os.getenv('EBVERSIONABRA2'))
        pysamstats_bin = os.path.join(dep_roots['pysamstats'], 'bin', 'pysamstats')

        regex_subs = [
            (r'^abra\s*=.*/abra.*.jar', 'abra = ' + abra2_jar),
            (r'^bedtools\s*=.*/bin/bedtools', 'bedtools = ' + os.path.join(dep_roots['BEDTools'], 'bin', 'bedtools')),
            (r'^flash\s*=.*/bin/flash2', 'flash = ' + os.path.join(dep_roots['FLASH'], 'bin', 'flash2')),
            (r'^java\s*=.*/bin/java', 'java = ' + os.path.join(dep_roots['Java'], 'bin', 'java')),
            (r'^prinseq\s*=.*/prinseq-lite.pl', 'prinseq = ' + os.path.join(dep_roots['PRINSEQ'], 'prinseq-lite.pl')),
            (r'^pysamstats\s*=.*/bin/pysamstats', 'pysamstats = ' + pysamstats_bin),
            (r'^rscript\s*=.*/bin/Rscript', 'rscript = ' + os.path.join(dep_roots['R'], 'bin', 'Rscript')),
            (r'^samtools\s*=.*/bin/samtools', 'samtools = ' + os.path.join(dep_roots['SAMtools'], 'bin', 'samtools')),
        ]

        for cfg_file in cfg_files:
            dirname = os.path.dirname(cfg_file)
            if os.path.basename(dirname).startswith('example'):
                example_dir = dirname
            else:
                example_dir = os.path.join(self.installdir, 'Examples', 'example1')

            genome_dir = os.path.join(example_dir, 'genome')
            genomex_fa = os.path.join(genome_dir, 'genomex.fa')

            regex_subs.extend([
                (r'^ref_fasta\s*=.*genome/genomex.fa', 'ref_fasta = ' + genomex_fa),
                (r'^bwa_idx\s*=.*genome/genomex.fa', 'bwa_idx = ' + genomex_fa),
                (r'^refGene\s*=.*genome/refgenex.txt', 'refGene = ' + os.path.join(genome_dir, 'refgenex.txt')),
            ])

            apply_regex_substitutions(cfg_file, regex_subs, on_missing_match='error')



##
# Copyright 2019-2023 Bart Oldeman, McGill University, Compute Canada
#
# This file is triple-licensed under GPLv2 (see below), MIT, and
# BSD three-clause licenses.
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for installing the Intel compiler suite, implemented as an easyblock

@author: Bart Oldeman (McGill University, Compute Canada)
"""

import os
from easybuild.easyblocks.icc import EB_icc
from easybuild.easyblocks.ifort import EB_ifort


class EB_iccifort(EB_ifort, EB_icc):
    """
    Class that can be used to install iccifort
    """

    def sanity_check_step(self):
        """Custom sanity check paths for iccifort."""
        EB_icc.sanity_check_step(self)
        EB_ifort.sanity_check_step(self)

    def make_module_extra(self):
        txt = super(EB_iccifort, self).make_module_extra()

        # also define $EBROOT* and $EBVERSION* for icc/ifort
        txt += self.module_generator.set_environment('EBROOTICC', self.installdir)
        txt += self.module_generator.set_environment('EBROOTIFORT', self.installdir)
        txt += self.module_generator.set_environment('EBVERSIONICC', self.version)
        txt += self.module_generator.set_environment('EBVERSIONIFORT', self.version)

        return txt

    def make_module_req_guess(self):
        # Use EB_icc because its make_module_req_guess deliberately omits 'include' for CPATH:
        # including it causes problems, e.g. with complex.h and std::complex
        # cfr. https://software.intel.com/en-us/forums/intel-c-compiler/topic/338378
        # whereas EB_ifort adds 'include' but that's only needed if icc and ifort are separate
        guesses = EB_icc.make_module_req_guess(self)

        # remove entries from LIBRARY_PATH that icc and co already know about at compile time
        # only do this for iccifort merged installations so that icc can still find ifort
        # libraries and vice versa for split installations
        if self.comp_libs_subdir is not None:
            compiler_library_paths = [os.path.join(self.comp_libs_subdir, p)
                                      for p in ('lib', 'compiler/lib/intel64', 'lib/ia32', 'lib/intel64')]
            guesses['LIBRARY_PATH'] = [p for p in guesses['LIBRARY_PATH'] if p not in compiler_library_paths]
        return guesses



# #
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
# #
"""
EasyBuild support for installing only the FFTW interfaces for the Intel Math Kernel Library (MKL),
implemented as an easyblock

@author: Bart Oldeman (McGill University, Calcul Quebec, Compute Canada)
"""
import os

from easybuild.easyblocks.imkl import EB_imkl
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.modules import get_software_root


class EB_imkl_minus_FFTW(EB_imkl):
    """
    Class that can be used to install mkl FFTW interfaces only
    """

    def prepare_step(self, *args, **kwargs):
        """Custom prepare step: make sure imkl is available as dependency."""
        super(EB_imkl_minus_FFTW, self).prepare_step(*args, **kwargs)

        imkl_root = get_software_root('imkl')
        if not imkl_root:
            raise EasyBuildError("Required imkl dependency is missing!")

    def install_step(self):
        """Install Intel MKL FFTW interfaces"""
        # correct mkl_basedir, since build of FFTW interfaces needs to be done from imkl install directory
        self.mkl_basedir = os.getenv('MKLROOT')
        self.build_mkl_fftw_interfaces(os.path.join(self.installdir, 'lib'))

    def make_module_req_guess(self):
        """Custom guesses for imkl-FFTW module file"""
        # bypass custom paths for imkl, only use standard library location
        return super(EB_imkl, self).make_module_req_guess()

    def make_module_extra(self):
        """Custom extra variables to set in module file"""
        # bypass extra module variables for imkl
        return super(EB_imkl, self).make_module_extra()

    def post_install_step(self):
        """Custom post install step for imkl-FFTW"""
        # bypass post_install_step of imkl easyblock
        pass

    def sanity_check_step(self):
        """Custom sanity check for imkl-FFTW: check if all libraries for FFTW interfaces are there."""
        custom_paths = {
            'files': [os.path.join(self.installdir, 'lib', x) for x in self.get_mkl_fftw_interface_libs()],
            'dirs': [],
        }
        super(EB_imkl, self).sanity_check_step(custom_paths=custom_paths)



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing RepeatModeler, implemented as an easyblock

@author: Jasper Grimm (UoY)
"""

import os

from easybuild.easyblocks.generic.tarball import Tarball
from easybuild.easyblocks.perl import get_site_suffix
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.filetools import change_dir, patch_perl_script_autoflush
from easybuild.tools.modules import get_software_root
from easybuild.tools.run import run_cmd_qa


def get_dep_path(dep_name, rel_path, log, optional):
    """
    Check for dependency, raise error if it's not available (unless optional=True is used).
    Return full path to specified relative path in dependency installation directory,
    return None if optional dependency is not found.
    """

    dep_root = get_software_root(dep_name)
    if dep_root:
        dep = os.path.join(dep_root, rel_path)
        return dep
    elif optional:
        log.info("Optional dependency not found: %s" % dep_name)
        return ''
    else:
        raise EasyBuildError("Missing required dependency: %s" % dep_name)


class EB_RepeatModeler(Tarball):
    """Support for building/installing RepeatModeler."""

    def install_step(self):
        """Custom install procedure for RepeatModeler."""
        super(EB_RepeatModeler, self).install_step()

        # Required dependencies, and bin path relative to software root
        required_deps = {
            'CD-HIT': 'bin',
            'Kent_tools': 'bin',
            'Perl': os.path.join('bin', 'perl'),
            'RECON': 'bin',
            'RepeatMasker': '',
            'RepeatScout': '',
            'TRF': os.path.join('bin', 'trf'),
        }

        # Search engines, and bin path relative to software root
        search_engines = {
            'ABBlast': 'bin',
            'RMBlast': 'bin',
            'WUBlast': 'bin',
        }

        # Optional dependencies for LTR pipeline, and bin path relative to software root
        optional_LTR_deps = {
            'GenomeTools': 'bin',
            'LTR_retriever': '',
            'MAFFT': 'bin',
            'TWL-NINJA': 'bin',
        }

        for dep, path in required_deps.items():
            required_deps[dep] = get_dep_path(dep, path, log=self.log, optional=False)

        for dep, path in search_engines.items():
            search_engines[dep] = get_dep_path(dep, path, log=self.log, optional=True)

        for dep, path in optional_LTR_deps.items():
            optional_LTR_deps[dep] = get_dep_path(dep, path, log=self.log, optional=True)

        # Check at least one search engine is present, and not both ABBlast/WUBlast
        if not any(search_engines.values()):
            raise EasyBuildError("At least one search engine must be specified: RMBlast, ABBlast, WUBlast")
        elif search_engines['ABBlast'] and search_engines['WUBlast']:
            raise EasyBuildError("Cannot specify both ABBlast and WUBlast")

        # Check if all LTR Pipeline dependencies present
        if all(optional_LTR_deps.values()):
            self.log.info("All LTR pipeline dependencies found, enabling LTR support")
            with_LTR = 'y'
        else:
            self.log.info("Not all LTR pipeline dependencies found, disabling LTR support")
            with_LTR = 'n'

        # Map search engine to configuration option
        search_engine_map = {
            'ABBlast': '2',
            'RMBlast': '1',
            'WUBlast': '2',
        }

        # List of search engines to use (mapped)
        mapped_engines = [search_engine_map[k] for k, v in search_engines.items() if v] + ['3']

        change_dir(self.installdir)

        # Fix perl shebang in configure script (#!/usr/local/bin/perl)
        orig_fix_perl_shebang_for = self.cfg['fix_perl_shebang_for']
        self.cfg['fix_perl_shebang_for'] = [os.path.join(self.installdir, 'configure')]
        self.fix_shebang()
        self.cfg['fix_perl_shebang_for'] = orig_fix_perl_shebang_for

        patch_perl_script_autoflush('configure')

        qa = {
            '<PRESS ENTER TO CONTINUE>': '',
        }
        std_qa = {
            r'\*\*PERL INSTALLATION PATH\*\*\n\n[^*]*\n+Enter path.*:\s*': required_deps['Perl'],
            r'UCSCTOOLS_DIR.*:\s*': required_deps['Kent_tools'],
            r'LTR_RETRIEVER_DIR.*:\s*': optional_LTR_deps['LTR_retriever'],
            r'RMBLAST_DIR.*:\s*': search_engines['RMBlast'],
            r'ABBLAST_DIR.*:\s*': search_engines['ABBlast'] or search_engines['WUBlast'],
            # Configure first engine
            r'.*(\[ Un\-configured \]\n.*){2}\n.*\n+Enter Selection\:\s*': mapped_engines[0],
            # Configure second engine if multiple specified, otherwise skip
            r'.*(\[ Un\-configured \]\n.*\[ Configured \]|\[ Configured \]\n.*\[ Un-configured \])'
            r'\n\n.*\n+Enter Selection\:\s*': mapped_engines[1],
            # All engines configured
            r'.*(\[ Configured \]\n.*){2}\n.*\n+Enter Selection\:\s*': '3',
            # LTR
            r'LTR.*\[optional](.*\n)*of analysis \[y] or n\?\:\s*': with_LTR,
        }

        cmdopts = ' ' + ' '.join([
            '-trf_prgm "%(TRF)s"',
            '-repeatmasker_dir "%(RepeatMasker)s"',
            '-rscout_dir "%(RepeatScout)s"',
            '-recon_dir "%(RECON)s"',
            '-ucsctools_dir "%(Kent_tools)s"',
            '-cdhit_dir "%(CD-HIT)s"',
        ]) % required_deps

        if with_LTR:
            cmdopts += ' ' + ' '.join([
                '-mafft_dir "%(MAFFT)s"',
                '-genometools_dir "%(GenomeTools)s"',
                '-ltr_retriever_dir "%(LTR_retriever)s"',
                '-ninja_dir "%(TWL-NINJA)s"',
            ]) % optional_LTR_deps

        cmd = "perl ./configure" + cmdopts
        run_cmd_qa(cmd, qa, std_qa=std_qa, log_all=True, simple=True, log_ok=True, maxhits=100)

    def sanity_check_step(self):
        """Custom sanity check for RepeatModeler."""

        custom_paths = {
            'files': ['BuildDatabase', 'RepeatClassifier', 'RepeatModeler', 'RepeatUtil.pm'],
            'dirs': ['Libraries', 'Matrices', 'util'],
        }

        custom_commands = [("RepeatModeler -help 2>&1 | grep 'RepeatModeler - Model repetitive DNA'", '')]

        super(EB_RepeatModeler, self).sanity_check_step(custom_commands=custom_commands, custom_paths=custom_paths)

    def make_module_req_guess(self):
        """Custom guesses for path-like environment variables for RepeatModelerConfig."""
        guesses = super(EB_RepeatModeler, self).make_module_req_guess()

        guesses.update({
            'PATH': [''],
            'PERL5LIB': [get_site_suffix('sitelib')],
        })

        return guesses



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for sympy, implemented as an easyblock

@author: Caspar van Leeuwen (SURF)
@author: Kenneth Hoste (HPC-UGent)
"""

import os
import tempfile

from easybuild.easyblocks.generic.pythonpackage import PythonPackage, det_pylibdir


class EB_sympy(PythonPackage):
    """Custom easyblock for installing the sympy Python package."""

    @staticmethod
    def extra_options(extra_vars=None):
        """Customize default value for easyconfig parameters for sympy"""
        extra_vars = PythonPackage.extra_options(extra_vars=extra_vars)
        extra_vars['use_pip'][0] = True
        extra_vars['sanity_pip_check'][0] = True
        extra_vars['download_dep_fail'][0] = True
        return extra_vars

    def test_step(self):
        """Custom test step for sympy"""

        self.cfg['runtest'] = "python setup.py test"

        # we need to make sure that the temporary directory being used is not a symlinked path;
        # see https://github.com/easybuilders/easybuild-easyconfigs/issues/17593
        original_tmpdir = tempfile.gettempdir()
        tempfile.tempdir = os.path.realpath(tempfile.gettempdir())
        msg = "Temporary directory set to resolved path %s (was %s), " % (original_tmpdir, tempfile.gettempdir())
        msg += "to avoid failing tests due to the temporary directory being a symlinked path..."
        self.log.info(msg)

        super(EB_sympy, self).test_step(self)

        # restore original temporary directory
        tempfile.tempdir = original_tmpdir
        self.log.debug("Temporary directory restored to %s", tempfile.gettempdir())

    def sanity_check_step(self, *args, **kwargs):
        """Custom sanity check for sympy."""

        # can't use self.pylibdir here, need to determine path on the fly using currently active 'python' command;
        # this is important for sympy installations for multiple Python version (via multi_deps)
        custom_paths = {
            'files': [os.path.join('bin', 'isympy')],
            'dirs': [os.path.join(det_pylibdir(), 'sympy')],
        }

        custom_commands = ["isympy --help"]

        return super(EB_sympy, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)



##
# Copyright 2013-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Libint, implemented as an easyblock

@author: Toon Verstraelen (Ghent University)
@author: Ward Poelmans (Ghent University)
"""

import os.path
from easybuild.tools import LooseVersion

from easybuild.easyblocks.generic.configuremake import DEFAULT_CONFIGURE_CMD, ConfigureMake
from easybuild.easyblocks.generic.cmakemake import CMakeMake
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.build_log import EasyBuildError, print_msg
from easybuild.tools.filetools import change_dir, extract_file
from easybuild.tools.run import run_cmd
from easybuild.tools.systemtools import get_shared_lib_ext


class EB_Libint(CMakeMake):

    @staticmethod
    def extra_options():
        """Custom easyconfig parameters for Libint."""
        extra_vars = {
            'libint_compiler_configopts': [True, "Configure options for Libint compiler", CUSTOM],
            'with_fortran': [False, "Enable Fortran support", CUSTOM],
        }
        return CMakeMake.extra_options(extra_vars)

    def configure_step(self):
        """Add some extra configure options."""

        if LooseVersion(self.version) >= LooseVersion('2.6.0'):
            # Libint 2.6.0 requires first compiling the Libint compiler,
            # by running configure with appropriate options, followed by 'make export'
            # and unpacking the resulting source tarball;
            # see https://github.com/evaleev/libint/wiki#compiling-libint-compiler

            # CMake is recommended, but configuring with Fortran support doesn't work correctly yet in Libint 2.6.0
            # so stick to traditional configure script for now
            print_msg("configuring Libint compiler...")

            # first run autogen.sh script to generate initial configure script
            run_cmd("./autogen.sh")

            cmd = ' '.join([
                self.cfg['preconfigopts'],
                './configure',
                self.cfg['configopts'],
                self.cfg['libint_compiler_configopts'],
            ])
            run_cmd(cmd)

            print_msg("generating Libint library...")
            run_cmd("make export")

            source_fn = 'libint-%s.tgz' % self.version
            if os.path.exists(source_fn):
                extract_file(source_fn, os.getcwd(), change_into_dir=False)
                change_dir('libint-%s' % self.version)
            else:
                raise EasyBuildError("Could not find generated source tarball after 'make export'!")

        # Libint < 2.7.0 can be configured using configure script,
        # Libint >= 2.7.0 should be configured via cmake
        if LooseVersion(self.version) < LooseVersion('2.7.0'):

            # also build shared libraries (not enabled by default)
            self.cfg.update('configopts', "--enable-shared")

            if self.toolchain.options['pic']:
                # Enforce consistency.
                self.cfg.update('configopts', "--with-pic")

            if LooseVersion(self.version) >= LooseVersion('2.0') and LooseVersion(self.version) < LooseVersion('2.1'):
                # the code in libint is automatically generated and hence it is in some
                # parts so complex that -O2 or -O3 compiler optimization takes forever
                self.cfg.update('configopts', "--with-cxx-optflags='-O1'")

            elif LooseVersion(self.version) >= LooseVersion('2.1'):
                # pass down $CXXFLAGS to --with-cxxgen-optflags configure option;
                # mainly to avoid warning about it not being set (but $CXXFLAGS is picked up anyway in practice)
                self.cfg.update('configopts', "--with-cxxgen-optflags='%s'" % os.getenv('CXXFLAGS'))

            # --enable-fortran is only a known configure option for Libint library, not for Libint compiler,
            # so only add --enable-fortran *after* configuring & generating Libint compiler
            if self.cfg['with_fortran']:
                self.cfg.update('configopts', '--enable-fortran')

            self.cfg['configure_cmd'] = DEFAULT_CONFIGURE_CMD
            ConfigureMake.configure_step(self)

        else:
            if self.cfg['with_fortran']:
                self.cfg.update('configopts', '-DENABLE_FORTRAN=ON')

            # also build shared libraries (not enabled by default)
            self.cfg.update('configopts', "-DLIBINT2_BUILD_SHARED_AND_STATIC_LIBS=ON")

            # specify current directory as source directory (that contains CMakeLists.txt),
            # since that's the path to the unpacked source tarball for Libint library (created by 'make export')
            super(EB_Libint, self).configure_step(srcdir=os.getcwd())

    def test_step(self):
        """Run Libint test suite for recent versions"""
        if LooseVersion(self.version) >= LooseVersion('2.1') and self.cfg['runtest'] is None:
            self.cfg['runtest'] = 'check'

        super(EB_Libint, self).test_step()

    def sanity_check_step(self):
        """Custom sanity check for Libint."""
        shlib_ext = get_shared_lib_ext()

        if LooseVersion(self.version) >= LooseVersion('2.0'):
            custom_paths = {
                'files': [os.path.join('lib', 'libint2.a'), os.path.join('lib', 'libint2.%s' % shlib_ext)],
                'dirs': [os.path.join('include', 'libint2')],
            }
            if LooseVersion(self.version) >= LooseVersion('2.1'):
                custom_paths['files'].extend([
                    os.path.join('include', 'libint2.h'),
                    os.path.join('include', 'libint2.hpp'),
                ])
                custom_paths['dirs'].extend([
                    os.path.join('share', 'libint'),
                    os.path.join('lib', 'pkgconfig'),
                ])
            else:
                custom_paths['files'].append(os.path.join('include', 'libint2', 'libint2.h'))

            if self.cfg['with_fortran']:
                custom_paths['files'].append(os.path.join('include', 'libint_f.mod'))
        else:
            headers = [os.path.join('include', 'libint', x) for x in ['libint.h', 'hrr_header.h', 'vrr_header.h']]
            libs = [os.path.join('lib', 'libint.a'), os.path.join('lib', 'libint.%s' % shlib_ext)]
            custom_paths = {
                'files': headers + libs,
                'dirs': [],
            }
        super(EB_Libint, self).sanity_check_step(custom_paths=custom_paths)

    def make_module_req_guess(self):
        """Specify correct CPATH for this installation."""
        guesses = super(EB_Libint, self).make_module_req_guess()
        if LooseVersion(self.version) >= LooseVersion('2.0'):
            libint_include = os.path.join('include', 'libint2')
        else:
            libint_include = os.path.join('include', 'libint')
        guesses.update({
            'CPATH': ['include', libint_include],
        })
        return guesses



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for installing MATLAB, implemented as an easyblock

@author: Stijn De Weirdt (Ghent University)
@author: Dries Verdegem (Ghent University)
@author: Kenneth Hoste (Ghent University)
@author: Pieter De Baets (Ghent University)
@author: Jens Timmerman (Ghent University)
@author: Fotis Georgatos (Uni.Lu, NTUA)
"""
import re
import os
import stat
import tempfile

from easybuild.tools import LooseVersion

from easybuild.easyblocks.generic.packedbinary import PackedBinary
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.filetools import adjust_permissions, change_dir, copy_file, read_file, write_file
from easybuild.tools.py2vs3 import string_type
from easybuild.tools.run import run_cmd


class EB_MATLAB(PackedBinary):
    """Support for installing MATLAB."""

    def __init__(self, *args, **kwargs):
        """Add extra config options specific to MATLAB."""
        super(EB_MATLAB, self).__init__(*args, **kwargs)
        self.comp_fam = None
        self.configfile = os.path.join(self.builddir, 'my_installer_input.txt')
        self.outputfile = os.path.join(self.builddir, 'my_installer_output.txt')

    @staticmethod
    def extra_options():
        extra_vars = {
            'java_options': ['-Xmx256m', "$_JAVA_OPTIONS value set for install and in module file.", CUSTOM],
            'key': [None, "Installation key(s), make one install for each key. Single key or a list of keys", CUSTOM],
        }
        return PackedBinary.extra_options(extra_vars)

    def configure_step(self):
        """Configure MATLAB installation: create license file."""

        licfile = self.cfg['license_file']
        if licfile is None:
            licserv = self.cfg['license_server']
            if licserv is None:
                licserv = os.getenv('EB_MATLAB_LICENSE_SERVER', 'license.example.com')
            licport = self.cfg['license_server_port']
            if licport is None:
                licport = os.getenv('EB_MATLAB_LICENSE_SERVER_PORT', '00000')
            # create license file
            lictxt = '\n'.join([
                "SERVER %s 000000000000 %s" % (licserv, licport),
                "USE_SERVER",
            ])

            licfile = os.path.join(self.builddir, 'matlab.lic')
            write_file(licfile, lictxt)

        try:
            copy_file(os.path.join(self.cfg['start_dir'], 'installer_input.txt'), self.configfile)
            adjust_permissions(self.configfile, stat.S_IWUSR)

            # read file in binary mode to avoid UTF-8 encoding issues when using Python 3,
            # due to non-UTF-8 characters...
            config = read_file(self.configfile, mode='rb')

            # use raw byte strings (must be 'br', not 'rb'),
            # required when using Python 3 because file was read in binary mode
            regdest = re.compile(br"^# destinationFolder=.*", re.M)
            regagree = re.compile(br"^# agreeToLicense=.*", re.M)
            regmode = re.compile(br"^# mode=.*", re.M)
            reglicpath = re.compile(br"^# licensePath=.*", re.M)
            regoutfile = re.compile(br"^# outputFile=.*", re.M)

            # must use byte-strings here when using Python 3, see above
            config = regdest.sub(b"destinationFolder=%s" % self.installdir.encode('utf-8'), config)
            config = regagree.sub(b"agreeToLicense=Yes", config)
            config = regmode.sub(b"mode=silent", config)
            config = reglicpath.sub(b"licensePath=%s" % licfile.encode('utf-8'), config)
            config = regoutfile.sub(b"outputFile=%s" % self.outputfile.encode('utf-8'), config)

            write_file(self.configfile, config)

        except IOError as err:
            raise EasyBuildError("Failed to create installation config file %s: %s", self.configfile, err)

        self.log.debug('configuration file written to %s:\n %s', self.configfile, config)

    def install_step(self):
        """MATLAB install procedure using 'install' command."""

        src = os.path.join(self.cfg['start_dir'], 'install')

        # make sure install script is executable
        adjust_permissions(src, stat.S_IXUSR)

        if LooseVersion(self.version) >= LooseVersion('2016b'):
            perm_dirs = [os.path.join(self.cfg['start_dir'], 'bin', 'glnxa64')]
            if LooseVersion(self.version) < LooseVersion('2021b'):
                jdir = os.path.join(self.cfg['start_dir'], 'sys', 'java', 'jre', 'glnxa64', 'jre', 'bin')
                perm_dirs.append(jdir)
            for perm_dir in perm_dirs:
                adjust_permissions(perm_dir, stat.S_IXUSR)

        # make sure $DISPLAY is not defined, which may lead to (hard to trace) problems
        # this is a workaround for not being able to specify --nodisplay to the install scripts
        if 'DISPLAY' in os.environ:
            os.environ.pop('DISPLAY')

        if '_JAVA_OPTIONS' not in self.cfg['preinstallopts']:
            java_opts = 'export _JAVA_OPTIONS="%s" && ' % self.cfg['java_options']
            self.cfg['preinstallopts'] = java_opts + self.cfg['preinstallopts']
        if LooseVersion(self.version) >= LooseVersion('2016b'):
            change_dir(self.builddir)

        # Build the cmd string
        cmdlist = [
            self.cfg['preinstallopts'],
            src,
            '-inputFile',
            self.configfile,
        ]
        if LooseVersion(self.version) < LooseVersion('2020a'):
            # MATLAB installers < 2020a ignore $TMPDIR (always use /tmp) and might need a large tmpdir
            tmpdir = tempfile.mkdtemp()
            cmdlist.extend([
                '-v',
                '-tmpdir',
                tmpdir,
            ])
        cmdlist.append(self.cfg['installopts'])
        cmd = ' '.join(cmdlist)

        keys = self.cfg['key']
        if keys is None:
            try:
                keys = os.environ['EB_MATLAB_KEY']
            except KeyError:
                raise EasyBuildError("The MATLAB install key is not set. This can be set either with the environment "
                                     "variable EB_MATLAB_KEY or by the easyconfig variable 'key'.")
        if isinstance(keys, string_type):
            keys = keys.split(',')

        # Compile the installation key regex outside of the loop
        regkey = re.compile(br"^(# )?fileInstallationKey=.*", re.M)

        # Run an install for each key
        for i, key in enumerate(keys):

            self.log.info('Installing MATLAB with key %s of %s', i + 1, len(keys))

            try:
                config = read_file(self.configfile, mode='rb')
                config = regkey.sub(b"fileInstallationKey=%s" % key.encode('utf-8'), config)
                write_file(self.configfile, config)

            except IOError as err:
                raise EasyBuildError("Failed to update config file %s: %s", self.configfile, err)

            (out, _) = run_cmd(cmd, log_all=True, simple=False)

            # check installer output for known signs of trouble
            patterns = [
                "Error: You have entered an invalid File Installation Key",
                "Not a valid key",
                "All selected products are already installed",
                "The application encountered an unexpected error and needs to close",
                "Error: Unable to write to",
                "Exiting with status -\\d",
                "End - Unsuccessful",
            ]

            for pattern in patterns:
                regex = re.compile(pattern, re.I)
                if regex.search(out):
                    raise EasyBuildError("Found error pattern '%s' in output of installation command '%s': %s",
                                         regex.pattern, cmd, out)
                with open(self.outputfile) as f:
                    if regex.search(f.read()):
                        raise EasyBuildError("Found error pattern '%s' in output file of installer",
                                             regex.pattern)

    def sanity_check_step(self):
        """Custom sanity check for MATLAB."""
        custom_paths = {
            'files': ["bin/matlab", "bin/glnxa64/MATLAB", "toolbox/local/classpath.txt"],
            'dirs': ["java/jar"],
        }
        super(EB_MATLAB, self).sanity_check_step(custom_paths=custom_paths)

    def make_module_extra(self):
        """Extend PATH and set proper _JAVA_OPTIONS (e.g., -Xmx)."""
        txt = super(EB_MATLAB, self).make_module_extra()

        if self.cfg['java_options']:
            txt += self.module_generator.set_environment('_JAVA_OPTIONS', self.cfg['java_options'])
        return txt



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for installing Mesa, implemented as an easyblock

@author: Andrew Edmondson (University of Birmingham)
@author: Kenneth Hoste (HPC-UGent)
@author: Alex Domingo (Vrije Universiteit Brussel)
@author: Alexander Grund (TU Dresden)
"""
import os
from easybuild.tools import LooseVersion

from easybuild.easyblocks.generic.mesonninja import MesonNinja
from easybuild.tools.filetools import copy_dir
from easybuild.tools.systemtools import POWER, X86_64, AARCH64
from easybuild.tools.systemtools import get_cpu_architecture, get_cpu_features, get_shared_lib_ext


class EB_Mesa(MesonNinja):
    """Custom easyblock for building and installing Mesa."""

    def __init__(self, *args, **kwargs):
        """Constructor for custom Mesa easyblock: figure out which values to pass to swr-arches configuration option."""

        super(EB_Mesa, self).__init__(*args, **kwargs)

        self.gallium_configopts = []

        # Mesa fails to build with libunwind on aarch64
        # See https://github.com/easybuilders/easybuild-easyblocks/issues/2150
        if get_cpu_architecture() == AARCH64:
            given_config_opts = self.cfg.get('configopts')
            if "-Dlibunwind=true" in given_config_opts:
                self.log.warning('libunwind not supported on aarch64, stripping from configopts!')
                configopts_libunwind_stripped = given_config_opts.replace('-Dlibunwind=true', '-Dlibunwind=false')
                self.cfg.set_keys({'configopts': configopts_libunwind_stripped})
                self.log.warning('New configopts after stripping: ' + self.cfg.get('configopts'))

        # Check user-defined Gallium drivers
        gallium_drivers = self.get_configopt_value('gallium-drivers')

        if not gallium_drivers:
            # Add appropriate Gallium drivers for current architecture
            arch = get_cpu_architecture()
            arch_gallium_drivers = {
                X86_64: ['swrast'],
                POWER: ['swrast'],
                AARCH64: ['swrast'],
            }
            if LooseVersion(self.version) < LooseVersion('22'):
                # swr driver support removed in Mesa 22.0
                arch_gallium_drivers[X86_64].append('swr')

            if arch in arch_gallium_drivers:
                gallium_drivers = arch_gallium_drivers[arch]
                # Add configopt for additional Gallium drivers
                self.gallium_configopts.append('-Dgallium-drivers=' + ','.join(gallium_drivers))

        self.log.debug('Gallium driver(s) included in the installation: %s' % ', '.join(gallium_drivers))

        self.swr_arches = []

        if 'swr' in gallium_drivers:
            # Check user-defined SWR arches
            self.swr_arches = self.get_configopt_value('swr-arches')

            if not self.swr_arches:
                # Set cpu features of SWR for current micro-architecture
                feat_to_swrarch = {
                    'avx': 'avx',
                    'avx1.0': 'avx',  # on macOS, AVX is indicated with 'avx1.0' rather than 'avx'
                    'avx2': 'avx2',
                    'avx512f': 'skx',  # AVX-512 Foundation - introduced in Skylake
                    'avx512er': 'knl',  # AVX-512 Exponential and Reciprocal Instructions implemented in Knights Landing
                }
                # Determine list of values to pass to swr-arches configuration option
                cpu_features = get_cpu_features()
                self.swr_arches = sorted([swrarch for feat, swrarch in feat_to_swrarch.items() if feat in cpu_features])
                # Add configopt for additional SWR arches
                self.gallium_configopts.append('-Dswr-arches=' + ','.join(self.swr_arches))

            self.log.debug('SWR Gallium driver will support: %s' % ', '.join(self.swr_arches))

    def get_configopt_value(self, configopt_name):
        """
        Return list of values for the given configuration option in configopts
        """
        configopt_args = [opt for opt in self.cfg['configopts'].split() if opt.startswith('-D%s=' % configopt_name)]

        if configopt_args:
            if len(configopt_args) > 1:
                self.log.warning("Found multiple instances of %s in configopts, using last one: %s",
                                 configopt_name, configopt_args[-1])
            # Get value of last option added
            configopt_value = configopt_args[-1].split('=')[-1]
            # Remove quotes and extract individual values
            configopt_value = configopt_value.strip('"\'').split(',')
        else:
            configopt_value = None

        return configopt_value

    def configure_step(self):
        """
        Customise the configure options based on the processor architecture of the host
        (Gallium drivers installed, SWR CPU features, ...)
        """

        if self.gallium_configopts:
            self.cfg.update('configopts', self.gallium_configopts)

        return super(EB_Mesa, self).configure_step()

    def install_step(self):
        """Also copy additional header files after installing Mesa."""

        super(EB_Mesa, self).install_step()

        # also install header files located in include/GL/internal, unless they're available already;
        # we can't enable both DRI and Gallium drivers,
        # but we can provide the DRI header file (GL/internal/dri_interface.h)
        target_inc_GL_internal = os.path.join(self.installdir, 'include', 'GL', 'internal')
        if not os.path.exists(target_inc_GL_internal):
            src_inc_GL_internal = os.path.join(self.start_dir, 'include', 'GL', 'internal')
            copy_dir(src_inc_GL_internal, target_inc_GL_internal)
            self.log.info("Copied %s to %s" % (src_inc_GL_internal, target_inc_GL_internal))

    def sanity_check_step(self):
        """Custom sanity check for Mesa."""

        shlib_ext = get_shared_lib_ext()

        if LooseVersion(self.version) >= LooseVersion('20.0'):
            header_files = [os.path.join('include', 'EGL', 'eglmesaext.h')]
            if LooseVersion(self.version) >= LooseVersion('22.3'):
                header_files.extend([os.path.join('include', 'EGL', 'eglext_angle.h')])
            else:
                header_files.extend([os.path.join('include', 'EGL', 'eglextchromium.h')])
            header_files.extend([
                os.path.join('include', 'GL', 'osmesa.h'),
                os.path.join('include', 'GL', 'internal', 'dri_interface.h'),
            ])
        else:
            gl_inc_files = ['glext.h', 'gl_mangle.h', 'glx.h', 'osmesa.h', 'gl.h', 'glxext.h', 'glx_mangle.h']
            gles_inc_files = [('GLES', 'gl.h'), ('GLES2', 'gl2.h'), ('GLES3', 'gl3.h')]
            header_files = [os.path.join('include', 'GL', x) for x in gl_inc_files]
            header_files.extend([os.path.join('include', x, y) for (x, y) in gles_inc_files])

        custom_paths = {
            'files': [os.path.join('lib', 'libOSMesa.%s' % shlib_ext)] + header_files,
            'dirs': [os.path.join('include', 'GL', 'internal')],
        }

        if self.swr_arches:
            swr_arch_libs = [os.path.join('lib', 'libswr%s.%s' % (a.upper(), shlib_ext)) for a in self.swr_arches]
            custom_paths['files'].extend(swr_arch_libs)

        super(EB_Mesa, self).sanity_check_step(custom_paths=custom_paths)

    def make_module_extra(self, *args, **kwargs):
        """ Append to EGL vendor library path,
        so that any NVidia libraries take precedence. """
        txt = super(EB_Mesa, self).make_module_extra(*args, **kwargs)
        # Append rather than prepend path to ensure that system NVidia drivers have priority.
        txt += self.module_generator.append_paths('__EGL_VENDOR_LIBRARY_DIRS', 'share/glvnd/egl_vendor.d')
        return txt



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for SCOTCH, implemented as an easyblock

@author: Stijn De Weirdt (Ghent University)
@author: Dries Verdegem (Ghent University)
@author: Kenneth Hoste (Ghent University)
@author: Pieter De Baets (Ghent University)
@author: Jens Timmerman (Ghent University)
"""
import os
from easybuild.tools import LooseVersion

import easybuild.tools.toolchain as toolchain
from easybuild.framework.easyblock import EasyBlock
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.filetools import apply_regex_substitutions, change_dir, copy_dir, copy_file
from easybuild.tools.filetools import remove_file, write_file
from easybuild.tools.run import run_cmd


class EB_SCOTCH(EasyBlock):
    """Support for building/installing SCOTCH."""

    @staticmethod
    def extra_options(extra_vars=None):
        """Define custom easyconfig parameters specific to Scotch."""
        extra_vars = {
            'threadedmpi': [None, "Use threaded MPI calls.", CUSTOM],
        }
        return EasyBlock.extra_options(extra_vars)

    def configure_step(self):
        """Configure SCOTCH build: locate the template makefile, copy it to a general Makefile.inc and patch it."""

        # pick template makefile
        comp_fam = self.toolchain.comp_family()
        if comp_fam == toolchain.INTELCOMP:  # @UndefinedVariable
            makefilename = 'Makefile.inc.x86-64_pc_linux2.icc'
        elif comp_fam == toolchain.GCC:  # @UndefinedVariable
            makefilename = 'Makefile.inc.x86-64_pc_linux2'
        else:
            raise EasyBuildError("Unknown compiler family used: %s", comp_fam)

        srcdir = os.path.join(self.cfg['start_dir'], 'src')

        # create Makefile.inc
        makefile_inc = os.path.join(srcdir, 'Makefile.inc')
        copy_file(os.path.join(srcdir, 'Make.inc', makefilename), makefile_inc)
        self.log.debug("Successfully copied Makefile.inc to src dir: %s", makefile_inc)

        # the default behaviour of these makefiles is still wrong
        # e.g., compiler settings, and we need -lpthread
        regex_subs = [
            (r"^CCS\s*=.*$", "CCS\t= $(CC)"),
            (r"^CCP\s*=.*$", "CCP\t= $(MPICC)"),
            (r"^CCD\s*=.*$", "CCD\t= $(MPICC)"),
            # append -lpthread to LDFLAGS
            (r"^LDFLAGS\s*=(?P<ldflags>.*$)", r"LDFLAGS\t=\g<ldflags> -lpthread"),
            # prepend -L${EBROOTZLIB}/lib to LDFLAGS
            (r"^LDFLAGS\s*=(?P<ldflags>.*$)", r"LDFLAGS\t=-L${EBROOTZLIB}/lib \g<ldflags>"),
        ]
        apply_regex_substitutions(makefile_inc, regex_subs)

        # change to src dir for building
        change_dir(srcdir)

    def build_step(self):
        """Build by running build_step, but with some special options for SCOTCH depending on the compiler."""

        ccs = os.environ['CC']
        ccp = os.environ['MPICC']
        ccd = os.environ['MPICC']

        cflags = "-fPIC -O3 -DCOMMON_FILE_COMPRESS_GZ -DCOMMON_PTHREAD -DCOMMON_RANDOM_FIXED_SEED -DSCOTCH_RENAME"
        if self.toolchain.comp_family() == toolchain.GCC:  # @UndefinedVariable
            cflags += " -Drestrict=__restrict"
        else:
            cflags += " -restrict -DIDXSIZE64"

        # USE 64 bit index
        if self.toolchain.options['i8']:
            cflags += " -DINTSIZE64"

        if self.cfg['threadedmpi']:
            cflags += " -DSCOTCH_PTHREAD"

        # For backwards compatability of v2.8.0 this is necessary but could be removed on a major version upgrade
        mpi_family = self.toolchain.mpi_family()
        if self.cfg['threadedmpi'] is None and mpi_family not in [toolchain.INTELMPI, toolchain.QLOGICMPI]:
            cflags += " -DSCOTCH_PTHREAD"

        # actually build
        apps = ['scotch', 'ptscotch']
        if LooseVersion(self.version) >= LooseVersion('6.0'):
            # separate target for esmumps in recent versions
            apps.extend(['esmumps', 'ptesmumps'])

        for app in apps:
            cmd = 'make CCS="%s" CCP="%s" CCD="%s" CFLAGS="%s" %s' % (ccs, ccp, ccd, cflags, app)
            run_cmd(cmd, log_all=True, simple=True)

    def install_step(self):
        """Install by copying files and creating group library file."""

        self.log.debug("Installing SCOTCH by copying files")

        for subdir in ['bin', 'include', 'lib', 'man']:
            copy_dir(os.path.join(self.cfg['start_dir'], subdir), os.path.join(self.installdir, subdir))

        # remove metis.h and parmetis.h include files, since those can only cause trouble
        for header in ['metis.h', 'parmetis.h']:
            remove_file(os.path.join(self.installdir, 'include', header))

        # create group library file
        scotchlibdir = os.path.join(self.installdir, 'lib')
        scotchgrouplib = os.path.join(scotchlibdir, 'libscotch_group.a')

        line = "GROUP (%s)" % ' '.join(os.listdir(scotchlibdir))
        write_file(scotchgrouplib, line)
        self.log.info("Successfully written group lib file: %s", scotchgrouplib)

    def sanity_check_step(self):
        """Custom sanity check for SCOTCH."""

        custom_paths = {
            'files': [],
            'dirs': [],
        }

        binaries = ['acpl', 'amk_ccc', 'amk_fft2', 'amk_grf', 'amk_hy', 'amk_m2', 'amk_p2', 'atst',
                    'dggath', 'dgmap', 'dgord', 'dgpart', 'dgscat', 'dgtst', 'gbase', 'gcv', 'gmap',
                    'gmk_hy', 'gmk_m2', 'gmk_m3', 'gmk_msh', 'gmk_ub2', 'gmtst', 'gord', 'gotst',
                    'gout', 'gpart', 'gscat', 'gtst', 'mcv', 'mmk_m2', 'mmk_m3', 'mord', 'mtst']
        custom_paths['files'].extend([os.path.join('bin', x) for x in binaries])

        headers = ['esmumps', 'ptscotch', 'ptscotchf', 'scotch', 'scotchf']
        custom_paths['files'].extend([os.path.join('include', '%s.h' % x) for x in headers])

        libraries = ['esmumps', 'ptesmumps', 'ptscotch', 'ptscotcherr', 'ptscotcherrexit',
                     'scotch', 'scotch_group', 'scotcherr', 'scotcherrexit']
        custom_paths['files'].extend([os.path.join('lib', 'lib%s.a' % x) for x in libraries])

        custom_commands = []

        # only hard check for recent SCOTCH versions;
        # older versions like '5.1.12b_esmumps' require more effort (not worth it)
        if LooseVersion(self.version) > LooseVersion('6.0.0'):
            # make sure installed SCOTCH version matches what we expect,
            # since it's easy to download the wrong tarball
            # cfr. https://github.com/easybuilders/easybuild-easyconfigs/issues/7154
            # (for '5.1.12b_esmumps', check with 'version 5.1.12')
            custom_commands.append("acpl -V 2>&1 | grep 'version %s'" % self.version.split('_')[0].strip('ab'))

        super(EB_SCOTCH, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)



##
# Copyright 2018-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Nim, implemented as an easyblock

author: Kenneth Hoste (HPC-UGent)
"""
import os

from easybuild.framework.easyblock import EasyBlock
from easybuild.tools.filetools import copy_file, move_file
from easybuild.tools.run import run_cmd


class EB_Nim(EasyBlock):
    """Support for building/installing Nim."""

    def configure_step(self):
        """No configuration for Nim."""
        pass

    def build_step(self):
        """Custom build procedure for Nim."""

        # build Nim (bin/nim)
        run_cmd("sh build.sh")

        # build koch management tool
        run_cmd("bin/nim c -d:release koch")

        # rebuild Nim, with readline bindings
        run_cmd("./koch boot -d:release -d:useLinenoise")

        # build nimble/nimgrep/nimsuggest tools
        run_cmd("./koch tools")

    def install_step(self):
        """Custom install procedure for Nim."""

        run_cmd("./koch geninstall")
        run_cmd("sh install.sh %s" % self.installdir)

        # install.sh copies stuff into <prefix>/nim, so move it
        nim_dir = os.path.join(self.installdir, 'nim')
        for entry in os.listdir(nim_dir):
            move_file(os.path.join(nim_dir, entry), os.path.join(self.installdir, entry))

        # also copy nimble/nimgrep/nimsuggest tools
        for tool in ['nimble', 'nimgrep', 'nimsuggest']:
            copy_file(os.path.join('bin', tool), os.path.join(self.installdir, 'bin', tool))

    def sanity_check_step(self):
        """Custom sanity check for Nim."""
        custom_paths = {
            'files': ['bin/nim', 'bin/nimble', 'bin/nimgrep', 'bin/nimsuggest'],
            'dirs': ['config', 'doc', 'lib'],
        }
        super(EB_Nim, self).sanity_check_step(custom_paths=custom_paths)



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Mamba, implemented as an easyblock

@author: Caspar van Leeuwen (SURF)
@author: Kenneth Hoste (HPC-UGent)
"""

import os

from easybuild.easyblocks.a.anaconda import EB_Anaconda


class EB_Mamba(EB_Anaconda):
    """Support for building/installing Mamba."""

    def sanity_check_step(self):
        """
        Custom sanity check for Mamba
        """
        custom_paths = {
            'files': [os.path.join('bin', x) for x in ['2to3', 'conda', 'pydoc', 'python', 'mamba']],
            'dirs': ['etc', 'lib', 'pkgs'],
        }
        # Directly call EB_Anaconda's super, as this sanity_check_step should _overwrite_ Anaconda's (not call it)
        super(EB_Anaconda, self).sanity_check_step(custom_paths=custom_paths)



##
# Copyright 2009-2023 Ghent University
# Copyright 2015-2023 Stanford University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://vscentrum.be/nl/en),
# the Hercules foundation (http://www.herculesstichting.be/in_English)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for Amber, implemented as an easyblock

Original author: Benjamin Roberts (The University of Auckland)
Modified by Stephane Thiell (Stanford University) for Amber14
Enhanced/cleaned up by Kenneth Hoste (HPC-UGent)
CMake support (Amber 20) added by James Carpenter and Simon Branford (University of Birmingham)
"""
from easybuild.tools import LooseVersion
import os

import easybuild.tools.environment as env
import easybuild.tools.toolchain as toolchain
from easybuild.easyblocks.generic.cmakemake import CMakeMake
from easybuild.easyblocks.generic.pythonpackage import det_pylibdir
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.modules import get_software_root
from easybuild.tools.run import run_cmd
from easybuild.tools.filetools import remove_dir, which


class EB_Amber(CMakeMake):
    """Easyblock for building and installing Amber"""

    @staticmethod
    def extra_options(extra_vars=None):
        """Extra easyconfig parameters specific to CMakeMake"""
        extra_vars = dict(CMakeMake.extra_options(extra_vars))
        extra_vars.update({
            # 'Amber': [True, "Build Amber in addition to AmberTools", CUSTOM],
            'patchlevels': ["latest", "(AmberTools, Amber) updates to be applied", CUSTOM],
            # The following is necessary because some patches to the Amber update
            # script update the update script itself, in which case it will quit
            # and insist on being run again. We don't know how many times will
            # be needed, but the number of times is patchlevel specific.
            'patchruns': [1, "Number of times to run Amber's update script before building", CUSTOM],
            # enable testing by default
            'runtest': [True, "Run tests after each build", CUSTOM],
            'static': [True, "Build statically linked executables", CUSTOM],
        })
        return CMakeMake.extra_options(extra_vars)

    def __init__(self, *args, **kwargs):
        """Easyblock constructor: initialise class variables."""
        super(EB_Amber, self).__init__(*args, **kwargs)

        if LooseVersion(self.version) < LooseVersion('20'):
            # Build Amber <20 in install directory
            self.build_in_installdir = True
            env.setvar('AMBERHOME', self.installdir)

        self.pylibdir = None

        self.with_cuda = False
        self.with_mpi = False

    def extract_step(self):
        """Extract sources; strip off parent directory during unpack"""
        self.cfg.update('unpack_options', "--strip-components=1")
        super(EB_Amber, self).extract_step()

    def patch_step(self, *args, **kwargs):
        """Patch Amber using 'update_amber' tool, prior to applying listed patch files (if any)."""

        # figure out which Python command to use to run the update_amber script;
        # by default it uses 'python', but this may not be available (on CentOS 8 for example);
        # note that the dependencies are not loaded yet at this point, so we're at the mercy of the OS here...
        python_cmd = None
        for cand_python_cmd in ['python', 'python3', 'python2']:
            if which(cand_python_cmd):
                python_cmd = cand_python_cmd
                break

        if python_cmd is None:
            raise EasyBuildError("No suitable Python command found to run update_amber script!")

        if self.cfg['patchlevels'] == "latest":
            cmd = "%s ./update_amber --update" % python_cmd
            # Run as many times as specified. It is the responsibility
            # of the easyconfig author to get this right, especially if
            # he or she selects "latest". (Note: "latest" is not
            # recommended for this reason and others.)
            for _ in range(self.cfg['patchruns']):
                run_cmd(cmd, log_all=True)
        else:
            for (tree, patch_level) in zip(['AmberTools', 'Amber'], self.cfg['patchlevels']):
                if patch_level == 0:
                    continue
                cmd = "%s ./update_amber --update-to %s/%s" % (python_cmd, tree, patch_level)
                # Run as many times as specified. It is the responsibility
                # of the easyconfig author to get this right.
                for _ in range(self.cfg['patchruns']):
                    run_cmd(cmd, log_all=True)

        super(EB_Amber, self).patch_step(*args, **kwargs)

    def configure_step(self):
        """Apply the necessary CMake config opts."""

        if LooseVersion(self.version) < LooseVersion('19'):
            # Configuring Amber <19 is done in install step.
            return

        # CMake will search a previous install directory for Amber-compiled libs. We will therefore
        # manually remove the install directory prior to configuration.
        remove_dir(self.installdir)

        external_libs_list = []

        mpiroot = get_software_root(self.toolchain.MPI_MODULE_NAME[0])
        if mpiroot and self.toolchain.options.get('usempi', None):
            self.with_mpi = True
            self.cfg.update('configopts', '-DMPI=TRUE')

        if self.toolchain.options.get('openmp', None):
            self.cfg.update('configopts', '-DOPENMP=TRUE')

        # note: for Amber 20, a patch is required to fix the CMake scripts so they're aware of FlexiBLAS:
        # - cmake/patched-cmake-modules/FindBLASFixed.cmake
        # - cmake/patched-cmake-modules/FindLAPACKFixed.cmake
        flexiblas_root = get_software_root('FlexiBLAS')
        if flexiblas_root:
            self.cfg.update('configopts', '-DBLA_VENDOR=FlexiBLAS')
        else:
            openblas_root = get_software_root('OpenBLAS')
            if openblas_root:
                self.cfg.update('configopts', '-DBLA_VENDOR=OpenBLAS')

        cudaroot = get_software_root('CUDA')
        if cudaroot:
            self.with_cuda = True
            self.cfg.update('configopts', '-DCUDA=TRUE')
            if get_software_root('NCCL'):
                self.cfg.update('configopts', '-DNCCL=TRUE')
                external_libs_list.append('nccl')

        pythonroot = get_software_root('Python')
        if pythonroot:
            self.cfg.update('configopts', '-DDOWNLOAD_MINICONDA=FALSE')
            self.cfg.update('configopts', '-DPYTHON_EXECUTABLE=%s' % os.path.join(pythonroot, 'bin', 'python'))

            self.pylibdir = det_pylibdir()
            pythonpath = os.environ.get('PYTHONPATH', '')
            env.setvar('PYTHONPATH', os.pathsep.join([os.path.join(self.installdir, self.pylibdir), pythonpath]))

        if get_software_root('FFTW'):
            external_libs_list.append('fftw')
        if get_software_root('netCDF'):
            external_libs_list.append('netcdf')
        if get_software_root('netCDF-Fortran'):
            external_libs_list.append('netcdf-fortran')
        if get_software_root('zlib'):
            external_libs_list.append('zlib')
        if get_software_root('Boost'):
            external_libs_list.append('boost')
        if get_software_root('PnetCDF'):
            external_libs_list.append('pnetcdf')

        # Force libs for available deps (see cmake/3rdPartyTools.cmake in Amber source for list of 3rd party libs)
        # This provides an extra layer of checking but should already be handled by TRUST_SYSTEM_LIBS=TRUE
        external_libs = ";".join(external_libs_list)
        self.cfg.update('configopts', "-DFORCE_EXTERNAL_LIBS='%s'" % external_libs)

        if get_software_root('FFTW') or get_software_root('imkl'):
            self.cfg.update('configopts', '-DUSE_FFT=TRUE')

        # Set standard compile options
        self.cfg.update('configopts', '-DCHECK_UPDATES=FALSE')
        self.cfg.update('configopts', '-DAPPLY_UPDATES=FALSE')
        self.cfg.update('configopts', '-DTRUST_SYSTEM_LIBS=TRUE')
        self.cfg.update('configopts', '-DCOLOR_CMAKE_MESSAGES=FALSE')

        # Amber recommend running the tests from the sources, rather than putting in installation dir
        # due to size. We handle tests under the install step
        self.cfg.update('configopts', '-DINSTALL_TESTS=FALSE')

        self.cfg.update('configopts', '-DCOMPILER=AUTO')

        # configure using cmake
        super(EB_Amber, self).configure_step()

    def build_step(self):
        """Build Amber"""
        if LooseVersion(self.version) < LooseVersion('20'):
            # Building Amber < 20 is done in install step.
            return

        super(EB_Amber, self).build_step()

    def test_step(self):
        """Testing Amber build is done in install step."""
        pass

    def configuremake_install_step(self):
        """Custom build, test & install procedure for Amber <20."""
        # unset $LIBS since it breaks the build
        env.unset_env_vars(['LIBS'])

        # define environment variables for MPI, BLAS/LAPACK & dependencies
        mklroot = get_software_root('imkl')
        flexiblas_root = get_software_root('FlexiBLAS')
        openblas_root = get_software_root('OpenBLAS')
        if mklroot:
            env.setvar('MKL_HOME', os.getenv('MKLROOT'))
        elif flexiblas_root or openblas_root:
            lapack = os.getenv('LIBLAPACK')
            if lapack is None:
                raise EasyBuildError("$LIBLAPACK for OpenBLAS or FlexiBLAS not defined in build environment!")
            else:
                env.setvar('GOTO', lapack)

        mpiroot = get_software_root(self.toolchain.MPI_MODULE_NAME[0])
        if mpiroot and self.toolchain.options.get('usempi', None):
            env.setvar('MPI_HOME', mpiroot)
            self.with_mpi = True
            if self.toolchain.mpi_family() == toolchain.INTELMPI:
                self.mpi_option = '-intelmpi'
            else:
                self.mpi_option = '-mpi'

        common_configopts = [self.cfg['configopts'], '--no-updates']

        if get_software_root('X11') is None:
            common_configopts.append('-noX11')

        if self.name == 'Amber' and self.cfg['static']:
            common_configopts.append('-static')

        netcdfroot = get_software_root('netCDF')
        if netcdfroot:
            common_configopts.extend(["--with-netcdf", netcdfroot])

        netcdf_fort_root = get_software_root('netCDF-Fortran')
        if netcdf_fort_root:
            common_configopts.extend(["--with-netcdf-fort", netcdf_fort_root])

        pythonroot = get_software_root('Python')
        if pythonroot:
            common_configopts.extend(["--with-python", os.path.join(pythonroot, 'bin', 'python')])

            self.pylibdir = det_pylibdir()
            pythonpath = os.environ.get('PYTHONPATH', '')
            env.setvar('PYTHONPATH', os.pathsep.join([os.path.join(self.installdir, self.pylibdir), pythonpath]))

        comp_fam = self.toolchain.comp_family()
        if comp_fam == toolchain.INTELCOMP:
            comp_str = 'intel'

        elif comp_fam == toolchain.GCC:
            comp_str = 'gnu'

        else:
            raise EasyBuildError("Don't know how to compile with compiler family '%s' -- check EasyBlock?", comp_fam)

        # The NAB compiles need openmp flag
        if self.toolchain.options.get('openmp', None):
            env.setvar('CUSTOMBUILDFLAGS', self.toolchain.get_flag('openmp'))

        # compose list of build targets
        build_targets = [('', 'test')]

        if self.with_mpi:
            build_targets.append((self.mpi_option, 'test.parallel'))
            # hardcode to 4 MPI processes, minimal required to run all tests
            env.setvar('DO_PARALLEL', self.toolchain.mpi_cmd_for('', 4))

        cudaroot = get_software_root('CUDA')
        if cudaroot:
            env.setvar('CUDA_HOME', cudaroot)
            self.with_cuda = True
            build_targets.append(('-cuda', 'test.cuda'))
            if self.with_mpi:
                build_targets.append(("-cuda %s" % self.mpi_option, 'test.cuda_parallel'))

        ld_lib_path = os.environ.get('LD_LIBRARY_PATH', '')
        env.setvar('LD_LIBRARY_PATH', os.pathsep.join([os.path.join(self.installdir, 'lib'), ld_lib_path]))

        for flag, testrule in build_targets:
            # configure
            cmd = "%s ./configure %s" % (self.cfg['preconfigopts'], ' '.join(common_configopts + [flag, comp_str]))
            (out, _) = run_cmd(cmd, log_all=True, simple=False)

            # build in situ using 'make install'
            # note: not 'build'
            super(EB_Amber, self).install_step()

            # test
            if self.cfg['runtest']:
                run_cmd("make %s" % testrule, log_all=True, simple=False)

            # clean, overruling the normal 'build'
            run_cmd("make clean")

    def install_step(self):
        """Install procedure for Amber."""

        if LooseVersion(self.version) < LooseVersion('20'):
            # pass into the configuremake build, install, and test method for Amber <20
            self.configuremake_install_step()
            return

        super(EB_Amber, self).install_step()

        # Run the tests located in the build directory
        if self.cfg['runtest']:
            pretestcommands = 'source %s/amber.sh && cd %s' % (self.installdir, self.builddir)

            # serial tests
            run_cmd("%s && make test.serial" % pretestcommands, log_all=True, simple=True)
            if self.with_cuda:
                (out, ec) = run_cmd("%s && make test.cuda_serial" % pretestcommands, log_all=True, simple=False)
                if ec > 0:
                    self.log.warning("Check the output of the Amber cuda tests for possible failures")

            if self.with_mpi:
                # Hard-code parallel tests to use 4 threads
                env.setvar("DO_PARALLEL", self.toolchain.mpi_cmd_for('', 4))
                (out, ec) = run_cmd("%s && make test.parallel" % pretestcommands, log_all=True, simple=False)
                if ec > 0:
                    self.log.warning("Check the output of the Amber parallel tests for possible failures")

            if self.with_mpi and self.with_cuda:
                # Hard-code CUDA parallel tests to use 2 threads
                env.setvar("DO_PARALLEL", self.toolchain.mpi_cmd_for('', 2))
                (out, ec) = run_cmd("%s && make test.cuda_parallel" % pretestcommands, log_all=True, simple=False)
                if ec > 0:
                    self.log.warning("Check the output of the Amber cuda_parallel tests for possible failures")

    def sanity_check_step(self):
        """Custom sanity check for Amber."""
        binaries = ['sander', 'tleap']
        if self.name == 'Amber':
            binaries.append('pmemd')
            if self.with_cuda:
                binaries.append('pmemd.cuda')
                if self.with_mpi:
                    if LooseVersion(self.version) < LooseVersion('20'):
                        binaries.append('pmemd.cuda.MPI')
                    else:
                        binaries.append('pmemd.cuda_DPFP.MPI')

        if self.with_mpi:
            binaries.extend(['sander.MPI'])
            if self.name == 'Amber':
                binaries.append('pmemd.MPI')

        custom_paths = {
            'files': [os.path.join(self.installdir, 'bin', binary) for binary in binaries],
            'dirs': [],
        }
        super(EB_Amber, self).sanity_check_step(custom_paths=custom_paths)

    def make_module_extra(self):
        """Add module entries specific to Amber/AmberTools"""
        txt = super(EB_Amber, self).make_module_extra()

        txt += self.module_generator.set_environment('AMBERHOME', self.installdir)
        if self.pylibdir:
            txt += self.module_generator.prepend_paths('PYTHONPATH', self.pylibdir)

        return txt



##
# Copyright 2013-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Mathematica, implemented as an easyblock

@author: Kenneth Hoste (Ghent University)
"""

from easybuild.tools import LooseVersion
import glob
import os

from easybuild.easyblocks.generic.binary import Binary
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.run import run_cmd_qa


class EB_Mathematica(Binary):
    """Support for building/installing Mathematica."""

    @staticmethod
    def extra_options():
        """Additional easyconfig parameters custom to Mathematica."""
        extra_vars = {
            'activation_key': [None, "Activation key (expected format: 0000-0000-AAAAA)", CUSTOM],
        }
        return Binary.extra_options(extra_vars)

    def configure_step(self):
        """No configuration for Mathematica."""
        # ensure a license server is specified
        if self.cfg['license_server'] is None:
            raise EasyBuildError("No license server specified.")

    def build_step(self):
        """No build step for Mathematica."""
        pass

    def install_step(self):
        """Install Mathematica using install script."""

        # make sure $DISPLAY is not set (to avoid that installer uses GUI)
        orig_display = os.environ.pop('DISPLAY', None)

        install_script_glob = '%s_%s_LINUX*.sh' % (self.name, self.version)
        # Starting at V13, Mathematica have renamed their install file...
        if LooseVersion(self.version) >= LooseVersion("13"):
            install_script_glob = '%s_%s_*LINUX*.sh' % (self.name, self.version)

        matches = glob.glob(install_script_glob)
        if len(matches) == 1:
            install_script = matches[0]
            cmd = self.cfg['preinstallopts'] + './' + install_script
            shortver = '.'.join(self.version.split('.')[:2])
            qa_install_path = os.path.join('/usr', 'local', 'Wolfram', self.name, shortver)
            qa = {
                r"Enter the installation directory, or press ENTER to select %s: >" % qa_install_path: self.installdir,
                r"Create directory (y/n)? >": 'y',
                r"Should the installer attempt to make this change (y/n)? >": 'n',
                r"or press ENTER to select /usr/local/bin: >": os.path.join(self.installdir, "bin"),
            }
            no_qa = [
                r"Now installing.*\n\n.*\[.*\].*",
            ]
            run_cmd_qa(cmd, qa, no_qa=no_qa, log_all=True, simple=True, maxhits=200)
        else:
            raise EasyBuildError("Failed to isolate install script using '%s': %s", install_script_glob, matches)

        # add license server configuration file
        # some relevant documentation at http://reference.wolfram.com/mathematica/tutorial/ConfigurationFiles.html
        mathpass_path = os.path.join(self.installdir, 'Configuration', 'Licensing', 'mathpass')
        try:
            # append to file, to avoid overwriting anything that might be there
            f = open(mathpass_path, "a")
            f.write("!%s\n" % self.cfg['license_server'])
            f.close()
            f = open(mathpass_path, "r")
            mathpass_txt = f.read()
            f.close()
            self.log.info("Updated license file %s: %s" % (mathpass_path, mathpass_txt))
        except IOError as err:
            raise EasyBuildError("Failed to update %s with license server info: %s", mathpass_path, err)

        # restore $DISPLAY if required
        if orig_display is not None:
            os.environ['DISPLAY'] = orig_display

    def post_install_step(self):
        """Activate installation by using activation key, if provided."""
        if self.cfg['activation_key']:
            # activation key is printed by using '$ActivationKey' in Mathematica, so no reason to keep it 'secret'
            self.log.info("Activating installation using provided activation key '%s'." % self.cfg['activation_key'])
            qa = {
                r"(enter return to skip Web Activation):": self.cfg['activation_key'],
                r"In[1]:= ": 'Quit[]',
            }
            noqa = [
                '^%s %s .*' % (self.name, self.version),
                '^Copyright.*',
            ]
            run_cmd_qa(os.path.join(self.installdir, 'bin', 'math'), qa, no_qa=noqa)
        else:
            self.log.info("No activation key provided, so skipping activation of the installation.")

        super(EB_Mathematica, self).post_install_step()

    def sanity_check_step(self):
        """Custom sanity check for Mathematica."""
        custom_paths = {
            'files': ['bin/mathematica'],
            'dirs': ['AddOns', 'Configuration', 'Documentation', 'Executables', 'SystemFiles'],
        }
        if LooseVersion(self.version) >= LooseVersion("11.3.0"):
            custom_paths['files'].append('Executables/wolframscript')
        elif LooseVersion(self.version) >= LooseVersion("11.0.0"):
            custom_paths['files'].append('bin/wolframscript')

        custom_commands = ['mathematica --version']

        super(EB_Mathematica, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)

    def make_module_req_guess(self):
        """Add both 'bin' and 'Executables' directories to PATH."""

        guesses = super(EB_Mathematica, self).make_module_req_guess()

        guesses.update({'PATH': ['bin', 'Executables']})

        return guesses



##
# Copyright 2018-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for installing a bundle of Perl modules, implemented as a generic easyblock

@author: Mikael Oehman (Chalmers University of Technology)
"""
import os

from easybuild.easyblocks.generic.bundle import Bundle
from easybuild.easyblocks.generic.perlmodule import PerlModule
from easybuild.easyblocks.perl import get_major_perl_version, get_site_suffix
from easybuild.tools.config import build_option
from easybuild.tools.environment import setvar


class PerlBundle(Bundle):
    """
    Bundle of perl modules
    """

    @staticmethod
    def extra_options():
        """Easyconfig parameters specific to bundles of Perl modules."""
        # combine custom easyconfig parameters of Bundle & PerlModule
        extra_vars = PerlModule.extra_options()
        return Bundle.extra_options(extra_vars)

    def __init__(self, *args, **kwargs):
        """Initialize PerlBundle easyblock."""
        super(PerlBundle, self).__init__(*args, **kwargs)

        self.cfg['exts_defaultclass'] = 'PerlModule'
        self.cfg['exts_filter'] = ("perl -e 'require %(ext_name)s'", '')

    def extensions_step(self, *args, **kwargs):
        """Install extensions"""

        setvar('INSTALLDIRS', 'site')
        # define $OPENSSL_PREFIX to ensure that e.g. Net-SSLeay extension picks up OpenSSL
        # from specified sysroot rather than from host OS
        sysroot = build_option('sysroot')
        if sysroot:
            setvar('OPENSSL_PREFIX', sysroot)

        super(PerlBundle, self).extensions_step(*args, **kwargs)

    def test_step(self):
        """No global test step for bundle of Perl modules."""
        # required since runtest is set to True by default
        pass

    def sanity_check_step(self, *args, **kwargs):
        """Custom sanity check for bundle of Perl modules."""

        if not self.cfg['sanity_check_paths']:
            majver = get_major_perl_version()
            self.cfg['sanity_check_paths'] = {
                'files': [],
                'dirs': [os.path.join('lib', 'perl%s' % majver)],
            }

        super(Bundle, self).sanity_check_step(*args, **kwargs)

    def make_module_extra(self):
        """Extra module entries for Perl bundles."""
        majver = get_major_perl_version()
        sitelibsuffix = get_site_suffix('sitelib')

        txt = super(Bundle, self).make_module_extra()
        txt += self.module_generator.prepend_paths("PERL%sLIB" % majver, [sitelibsuffix])
        return txt



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing NCL, implemented as an easyblock

@author: Stijn De Weirdt (Ghent University)
@author: Dries Verdegem (Ghent University)
@author: Kenneth Hoste (Ghent University)
@author: Pieter De Baets (Ghent University)
@author: Jens Timmerman (Ghent University)
"""

import fileinput
import os
import re
import sys

from easybuild.framework.easyblock import EasyBlock
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.modules import get_software_root
from easybuild.tools.run import run_cmd


class EB_NCL(EasyBlock):
    """Support for building/installing NCL."""

    def configure_step(self):
        """Configure build:
        - create Makefile.ini using make and run ymake script to create config file
        - patch config file with correct settings, and add missing config entries
        - create config/Site.local file to avoid interactive install
        - generate Makefile using config/ymkmf sciprt
        -
        """

        try:
            os.chdir('config')
        except OSError as err:
            raise EasyBuildError("Failed to change to the 'config' dir: %s", err)

        cmd = "make -f Makefile.ini"
        run_cmd(cmd, log_all=True, simple=True)

        cmd = "./ymake -config $PWD"
        run_cmd(cmd, log_all=True, simple=True)

        # figure out name of config file
        cfg_regexp = re.compile(r'^\s*SYSTEM_INCLUDE\s*=\s*"(.*)"\s*$', re.M)
        f = open("Makefile", "r")
        txt = f.read()
        f.close()
        cfg_filename = cfg_regexp.search(txt).group(1)

        # adjust config file as needed
        ctof_libs = ''
        ifort = get_software_root('ifort')
        if ifort:
            if os.path.exists('%s/lib/intel64/' % ifort) and os.access('%s/lib/intel64/' % ifort, os.R_OK):
                ctof_libs += '-lm -L%s/lib/intel64/ -lifcore -lifport' % ifort
            else:
                self.log.warning(
                    "Can't find a libdir for ifortran libraries -lifcore -lifport: "
                    "%s/lib/intel64 doesn't exist or is not accessible." % ifort
                )
        elif get_software_root('GCC'):
            ctof_libs = '-lgfortran -lm'

        macrodict = {
            'CCompiler': os.getenv('CC'),
            'CxxCompiler': os.getenv('CXX'),
            'FCompiler': os.getenv('F90'),
            'CcOptions': '-ansi %s' % os.getenv('CFLAGS'),
            'FcOptions': os.getenv('FFLAGS'),
            'COptimizeFlag': os.getenv('CFLAGS'),
            'FOptimizeFlag': os.getenv('FFLAGS'),
            'ExtraSysLibraries': os.getenv('LDFLAGS'),
            'CtoFLibraries': ctof_libs,
        }

        # replace config entries that are already there
        for line in fileinput.input(cfg_filename, inplace=1, backup='%s.orig' % cfg_filename):
            for (key, val) in list(macrodict.items()):
                regexp = re.compile(r"(#define %s\s*).*" % key)
                match = regexp.search(line)
                if match:
                    line = "#define %s %s\n" % (key, val)
                    macrodict.pop(key)
            sys.stdout.write(line)

        # add remaining config entries
        f = open(cfg_filename, "a")
        for (key, val) in macrodict.items():
            f.write("#define %s %s\n" % (key, val))
        f.close()

        f = open(cfg_filename, "r")
        self.log.debug("Contents of %s: %s" % (cfg_filename, f.read()))
        f.close()

        # configure
        try:
            os.chdir(self.cfg['start_dir'])
        except OSError as err:
            raise EasyBuildError("Failed to change to the build dir %s: %s", self.cfg['start_dir'], err)

        # instead of running the Configure script that asks a zillion questions,
        # let's just generate the config/Site.local file ourselves...

        # order of deps is important
        # HDF needs to go after netCDF, because both have a netcdf.h include file
        deps = ["HDF5", "JasPer", "netCDF", "HDF", "g2lib", "g2clib", "Szip", "UDUNITS"]

        libs = ''
        includes = ''
        for dep in deps:
            root = get_software_root(dep)
            if not root:
                raise EasyBuildError("%s not available", dep)

            # try %s/lib, if it doesn't exist, try %s/lib64
            if os.path.exists('%s/lib' % root) and os.access('%s/lib' % root, os.R_OK):
                libs += ' -L%s/lib ' % root
            elif os.path.exists('%s/lib64' % root) and os.access('%s/lib64' % root, os.R_OK):
                libs += ' -L%s/lib64 ' % root
            else:
                self.log.warning(
                    "Can't find a libdir for dependency %s: %s/lib and %s/lib64 don't exist." % (dep, root, root)
                )

            if os.path.exists('%s/include' % root) and os.access('%s/include' % root, os.R_OK):
                includes += ' -I%s/include ' % root
            else:
                self.log.warning("Can't find an include dir for dependency %s: %s/include doesn't exist." % (dep, root))

        opt_deps = ["netCDF-Fortran", "GDAL"]
        libs_map = {
            'netCDF-Fortran': '-lnetcdff -lnetcdf',
            'GDAL': '-lgdal',
        }
        for dep in opt_deps:
            root = get_software_root(dep)
            if root:
                # try %s/lib, if it doesn't exist, try %s/lib64
                if os.path.exists('%s/lib' % root) and os.access('%s/lib' % root, os.R_OK):
                    libs += ' -L%s/lib %s ' % (root, libs_map[dep])
                elif os.path.exists('%s/lib64' % root) and os.access('%s/lib64' % root, os.R_OK):
                    libs += ' -L%s/lib64 %s ' % (root, libs_map[dep])
                else:
                    self.log.warning(
                        "Can't find a libdir for dependency %s: %s/lib and %s/lib64 don't exist." % (dep, root, root)
                    )

                if os.path.exists('%s/include' % root) and os.access('%s/include' % root, os.R_OK):
                    includes += ' -I%s/include ' % root
                    self.log.warning(
                        "Can't find an include dir for dependency %s: %s/include doesn't exist." % (dep, root)
                    )

        cfgtxt = """#ifdef FirstSite
#endif /* FirstSite */

#ifdef SecondSite

#define YmakeRoot %(installdir)s

#define LibSearch %(libs)s
#define IncSearch %(includes)s

#define BuildNCL 1
#define HDFlib
#define HDFEOSlib
#define BuildGRIB2 1
#define BuildESMF 1

#define UdUnitslib -ludunits2

#define BuildRasterHDF 0
#define BuildHDF4 0
#define BuildTRIANGLE 0
#define BuildHDFEOS 0
#define BuildHDFEOS5 0

#endif /* SecondSite */
""" % {
            'installdir': self.installdir,
            'libs': libs,
            'includes': includes
        }

        f = open("config/Site.local", "w")
        f.write(cfgtxt)
        f.close()

        # generate Makefile
        cmd = "./config/ymkmf"
        run_cmd(cmd, log_all=True, simple=True)

    def build_step(self):
        """Building is done in install_step."""
        pass

    def install_step(self):
        """Build in install dir using build_step."""

        cmd = "%s make Everything %s" % (self.cfg['preinstallopts'], self.cfg['installopts'])
        run_cmd(cmd, log_all=True, simple=True)

    def sanity_check_step(self):
        """
        Custom sanity check for NCL
        """
        custom_paths = {
            'files': ['bin/fontc', 'bin/ncl', 'lib/libncl.a', 'lib/libncarg.a'],
            'dirs': ['include/ncarg', 'lib/ncarg/fontcaps'],
        }
        super(EB_NCL, self).sanity_check_step(custom_paths=custom_paths)

    def make_module_extra(self):
        """Set NCARG_ROOT environment variable in module."""
        txt = super(EB_NCL, self).make_module_extra()
        txt += self.module_generator.set_environment('NCARG_ROOT', self.installdir)
        return txt



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for installing the Intel Advisor XE, implemented as an easyblock

@author: Lumir Jasiok (IT4Innovations)
@author: Damian Alvarez (Forschungszentrum Juelich GmbH)
@author: Josef Dvoracek (Institute of Physics, Czech Academy of Sciences)
"""

import os
from easybuild.tools import LooseVersion

from easybuild.easyblocks.generic.intelbase import IntelBase


class EB_Advisor(IntelBase):
    """
    Support for installing Intel Advisor XE
    """

    def __init__(self, *args, **kwargs):
        """Constructor, initialize class variables."""
        super(EB_Advisor, self).__init__(*args, **kwargs)
        if LooseVersion(self.version) < LooseVersion('2017'):
            self.subdir = 'advisor_xe'
        elif LooseVersion(self.version) < LooseVersion('2021'):
            self.subdir = 'advisor'
        else:
            self.subdir = os.path.join('advisor', 'latest')

    def prepare_step(self, *args, **kwargs):
        """Since 2019u3 there is no license required."""
        if LooseVersion(self.version) >= LooseVersion('2019_update3'):
            kwargs['requires_runtime_license'] = False
        super(EB_Advisor, self).prepare_step(*args, **kwargs)

    def make_module_req_guess(self):
        """Find reasonable paths for Advisor"""
        return self.get_guesses_tools()

    def sanity_check_step(self):
        """Custom sanity check paths for Advisor"""
        binaries = ['advixe-cl', 'advixe-feedback', 'advixe-gui', 'advixe-runss', 'advixe-runtrc', 'advixe-runtc']
        custom_paths = self.get_custom_paths_tools(binaries)
        super(EB_Advisor, self).sanity_check_step(custom_paths=custom_paths)



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for installing the Intel Threading Building Blocks (TBB) library, implemented as an easyblock

@author: Stijn De Weirdt (Ghent University)
@author: Dries Verdegem (Ghent University)
@author: Kenneth Hoste (Ghent University)
@author: Pieter De Baets (Ghent University)
@author: Jens Timmerman (Ghent University)
@author: Lumir Jasiok (IT4Innovations)
@author: Simon Branford (University of Birmingham)
"""

import glob
import os
import shutil
from easybuild.tools import LooseVersion

from easybuild.easyblocks.generic.configuremake import ConfigureMake
from easybuild.easyblocks.generic.intelbase import INSTALL_MODE_NAME_2015, INSTALL_MODE_2015
from easybuild.easyblocks.generic.intelbase import IntelBase, ACTIVATION_NAME_2012, LICENSE_FILE_NAME_2012
from easybuild.framework.easyconfig import CUSTOM
from easybuild.tools.filetools import find_glob_pattern, move_file, symlink
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.modules import get_software_version
from easybuild.tools.systemtools import POWER, get_cpu_architecture, get_gcc_version, get_platform_name
from easybuild.tools.run import run_cmd


def get_tbb_gccprefix(libpath):
    """
    Find the correct gcc version for the lib dir of TBB
    """
    # using get_software_version('GCC') won't work if the system toolchain is used
    gccversion = get_software_version('GCCcore') or get_software_version('GCC')
    # manual approach to at least have the system version of gcc
    if not gccversion:
        gccversion = get_gcc_version()

    # TBB directory structure
    # https://www.threadingbuildingblocks.org/docs/help/tbb_userguide/Linux_OS.html
    tbb_gccprefix = 'gcc4.4'  # gcc version 4.4 or higher that may or may not support exception_ptr
    if gccversion:
        gccversion = LooseVersion(gccversion)
        if gccversion >= LooseVersion("4.1") and gccversion < LooseVersion("4.4"):
            tbb_gccprefix = 'gcc4.1'  # gcc version number between 4.1 and 4.4 that do not support exception_ptr
        elif os.path.isdir(os.path.join(libpath, 'gcc4.8')) and gccversion >= LooseVersion("4.8"):
            tbb_gccprefix = 'gcc4.8'

    return tbb_gccprefix


class EB_tbb(IntelBase, ConfigureMake):
    """EasyBlock for tbb, threading building blocks"""

    @staticmethod
    def extra_options():
        extra_vars = IntelBase.extra_options()
        extra_vars.update(ConfigureMake.extra_options())
        extra_vars.update({
            'with_python': [False, "Should the TBB4Python bindings be built as well?", CUSTOM],
        })
        return extra_vars

    def __init__(self, *args, **kwargs):
        """Initialisation of custom class variables for tbb"""
        super(EB_tbb, self).__init__(*args, **kwargs)

        platform_name = get_platform_name()
        myarch = get_cpu_architecture()
        if platform_name.startswith('x86_64'):
            self.arch = "intel64"
        elif platform_name.startswith('i386') or platform_name.startswith('i686'):
            self.arch = 'ia32'
        elif myarch == POWER:
            self.arch = 'ppc64'
        else:
            raise EasyBuildError("Failed to determine system architecture based on %s", platform_name)

        if not self.toolchain.is_system_toolchain():
            # open-source TBB version
            self.build_in_installdir = True
            self.cfg['requires_runtime_license'] = False

        if self.toolchain.is_system_toolchain():
            self.tbb_subdir = 'tbb'
        else:
            self.tbb_subdir = ''

    def extract_step(self):
        """Extract sources."""
        if not self.toolchain.is_system_toolchain():
            # strip off 'tbb-<version>' subdirectory
            self.cfg['unpack_options'] = "--strip-components=1"
        super(EB_tbb, self).extract_step()

    def configure_step(self):
        """Configure TBB build/installation."""
        if self.toolchain.is_system_toolchain():
            IntelBase.configure_step(self)
        else:
            # no custom configure step when building from source
            pass

    def build_step(self):
        """Configure TBB build/installation."""
        if self.toolchain.is_system_toolchain():
            IntelBase.build_step(self)
        else:
            # build with: make compiler={icl, icc, gcc, clang}
            self.cfg.update('buildopts', 'compiler="%s"' % os.getenv('CC'))
            ConfigureMake.build_step(self)

            if self.cfg['with_python']:
                # Uses the Makefile target `python`
                self.cfg.update('buildopts', 'python')
                ConfigureMake.build_step(self)

    def _has_cmake(self):
        """Check if CMake is included in the build deps"""
        build_deps = self.cfg.dependencies(build_only=True)
        return any(dep['name'] == 'CMake' for dep in build_deps)

    def install_step(self):
        """Custom install step, to add extra symlinks"""
        install_tbb_lib_path = os.path.join(self.installdir, 'tbb', 'lib')

        if self.toolchain.is_system_toolchain():
            silent_cfg_names_map = None
            silent_cfg_extras = None

            if LooseVersion(self.version) < LooseVersion('4.2'):
                silent_cfg_names_map = {
                    'activation_name': ACTIVATION_NAME_2012,
                    'license_file_name': LICENSE_FILE_NAME_2012,
                }

            elif LooseVersion(self.version) < LooseVersion('4.4'):
                silent_cfg_names_map = {
                    'install_mode_name': INSTALL_MODE_NAME_2015,
                    'install_mode': INSTALL_MODE_2015,
                }

            # In case of TBB 4.4.x and newer we have to specify ARCH_SELECTED in silent.cfg
            if LooseVersion(self.version) >= LooseVersion('4.4'):
                silent_cfg_extras = {
                    'ARCH_SELECTED': self.arch.upper()
                }

            IntelBase.install_step(self, silent_cfg_names_map=silent_cfg_names_map, silent_cfg_extras=silent_cfg_extras)

            # determine libdir
            libpath = os.path.join(self.installdir, 'tbb', 'libs', 'intel64')
            if LooseVersion(self.version) < LooseVersion('4.1.0'):
                libglob = os.path.join(libpath, 'cc*libc*_kernel*')
                libs = sorted(glob.glob(libglob), key=LooseVersion)
                if libs:
                    # take the last one, should be ordered by cc version
                    # we're only interested in the last bit
                    libpath = libs[-1]
                else:
                    raise EasyBuildError("No libs found using %s in %s", libglob, self.installdir)
            else:
                libpath = os.path.join(libpath, get_tbb_gccprefix(libpath))

            # applications go looking into tbb/lib so we move what's in there to tbb/libs
            shutil.move(install_tbb_lib_path, os.path.join(self.installdir, 'tbb', 'libs'))
            # And add a symlink of the library folder to tbb/lib
            symlink(libpath, install_tbb_lib_path)
        else:
            # no custom install step when building from source (building is done in install directory)
            libpath = find_glob_pattern(os.path.join(self.installdir, 'build', '*_release'))

        real_libpath = os.path.realpath(libpath)
        self.log.debug("libpath: %s, resolved: %s" % (libpath, real_libpath))
        libpath = real_libpath
        # applications usually look into /lib, so we move the folder there
        # This is also important so that /lib and /lib64 are actually on the same level
        root_lib_path = os.path.join(self.installdir, 'lib')
        move_file(libpath, root_lib_path)
        # Create a relative symlink at the original location as-if we didn't move it.
        # Note that the path must be relative from the folder where the symlink will be!
        symlink(os.path.relpath(root_lib_path, os.path.dirname(libpath)), libpath, use_abspath_source=False)

        # Install CMake config files if possible
        if self._has_cmake() and LooseVersion(self.version) >= LooseVersion('2020.0'):
            cmake_install_dir = os.path.join(root_lib_path, 'cmake', 'TBB')
            cmd = [
                'cmake',
                '-DINSTALL_DIR=' + cmake_install_dir,
                '-DSYSTEM_NAME=Linux',
                '-P tbb_config_installer.cmake',
            ]
            run_cmd(' '.join(cmd), path=os.path.join(self.builddir, 'cmake'))

    def sanity_check_step(self):
        """Custom sanity check for TBB"""
        custom_paths = {
            'files': [
                os.path.join('lib', 'libtbb.so'),
                os.path.join('lib', 'libtbbmalloc.so'),
            ],
            'dirs': [],
        }
        custom_commands = []

        if self.toolchain.is_system_toolchain():
            custom_paths['dirs'].extend(os.path.join('tbb', p) for p in
                                        ('bin', 'lib', 'libs', os.path.join('include', 'tbb')))
            custom_paths['files'].extend([
                os.path.join('tbb', 'lib', 'libtbb.so'),
                os.path.join('tbb', 'lib', 'libtbbmalloc.so'),
            ])
        else:
            custom_paths['dirs'].append(os.path.join('include', 'tbb'))

        if self._has_cmake():
            custom_paths['files'].extend([
                os.path.join('lib', 'cmake', 'TBB', 'TBBConfig.cmake'),
                os.path.join('lib', 'cmake', 'TBB', 'TBBConfigVersion.cmake'),
            ])

        if self.cfg['with_python']:
            custom_paths['dirs'].append(os.path.join(self.tbb_subdir, 'python'))
            custom_commands.extend(['python -c "import tbb"'])

        super(EB_tbb, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)

    def make_module_extra(self):
        """Add correct path to lib to LD_LIBRARY_PATH. and intel license file"""
        txt = super(EB_tbb, self).make_module_extra()

        if self.toolchain.is_system_toolchain():
            txt += self.module_generator.prepend_paths('CPATH', [os.path.join(self.tbb_subdir, 'include')])

        root_dir = os.path.join(self.installdir, self.tbb_subdir)
        txt += self.module_generator.set_environment('TBBROOT', root_dir)
        # TBB_ROOT used e.g. by FindTBB.cmake
        txt += self.module_generator.set_environment('TBB_ROOT', root_dir)

        if self.cfg['with_python']:
            txt += self.module_generator.prepend_paths('PYTHONPATH', [os.path.join(self.tbb_subdir, 'python')])

        return txt

    def cleanup_step(self):
        """Cleanup step"""
        if self.toolchain.is_system_toolchain():
            IntelBase.cleanup_step(self)
        else:
            ConfigureMake.cleanup_step(self)



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing Doxygen, implemented as an easyblock

@author: Stijn De Weirdt (Ghent University)
@author: Dries Verdegem (Ghent University)
@author: Kenneth Hoste (Ghent University)
@author: Pieter De Baets (Ghent University)
@author: Jens Timmerman (Ghent University)
@author: Balazs Hajgato (Free University Brussels (VUB))
"""

from easybuild.tools import LooseVersion
from easybuild.tools.run import run_cmd
from easybuild.easyblocks.generic.cmakemake import CMakeMake


class EB_Doxygen(CMakeMake):
    """Support for building/installing Doxygen"""

    def configure_step(self):
        """Configure build using non-standard configure prefix option (without `=`)
           for versions before 1.8.10. Newer versions use cmake instead of configure"""
        if LooseVersion(self.version) < LooseVersion("1.8.10"):

            cmd = "%s ./configure --prefix %s %s" % (self.cfg['preconfigopts'], self.installdir,
                                                     self.cfg['configopts'])
            run_cmd(cmd, log_all=True, simple=True)
        else:
            super(EB_Doxygen, self).configure_step()

    def sanity_check_step(self):
        """Custom sanity check for Doxygen"""

        custom_paths = {
            'files': ["bin/doxygen"],
            'dirs': []
        }

        custom_commands = ["doxygen --help"]

        super(EB_Doxygen, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)



##
# Copyright 2023 Utrecht University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for building and installing flook, implemented as an easyblock

@author: Arnold Kole (Utrecht University)
"""

from easybuild.easyblocks.generic.configuremake import ConfigureMake


class EB_flook(ConfigureMake):
    """Support for building/installing flook."""

    def __init__(self, *args, **kwargs):
        # call out to original constructor first, so 'self' (i.e. the class instance) is initialised
        super(EB_flook, self).__init__(*args, **kwargs)

        # Determine vendor
        vendor = None
        if self.toolchain.COMPILER_FAMILY == 'Clang':
            vendor = 'clang'
        elif self.toolchain.COMPILER_FAMILY == 'GCC':
            vendor = 'gnu'
        elif self.toolchain.COMPILER_FAMILY == 'Intel':
            vendor = 'intel'
        elif self.toolchain.COMPILER_FAMILY == 'PGI':
            vendor = 'pgi'

        # Set some default options
        if vendor is not None:
            local_comp_flags = 'VENDOR="%s" FFLAGS="$FFLAGS" CFLAGS="$CFLAGS"' % vendor
        else:
            local_comp_flags = 'FFLAGS="$FFLAGS" CFLAGS="$CFLAGS"'
        self.cfg.update('buildopts', 'liball %s' % local_comp_flags)
        self.cfg['parallel'] = 1

    def configure_step(self):
        # flook has no configure step
        pass

    def install_step(self):
        self.cfg.update('install_cmd', 'PREFIX=%s' % self.installdir)
        super(EB_flook, self).install_step()

    def sanity_check_step(self):
        custom_paths = {
            'files': ['include/flook.mod', 'lib/libflook.a', 'lib/libflookall.a', 'lib/pkgconfig/flook.pc'],
            'dirs': [],
        }

        # call out to parent to do the actual sanity checking, pass through custom paths
        super(EB_flook, self).sanity_check_step(custom_paths=custom_paths)



##
# Copyright 2012-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBlock for installing Java, implemented as an easyblock

@author: Jens Timmerman (Ghent University)
@author: Kenneth Hoste (Ghent University)
"""
import glob
import os
import stat

from easybuild.tools import LooseVersion
from easybuild.easyblocks.generic.packedbinary import PackedBinary
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.config import build_option
from easybuild.tools.filetools import adjust_permissions, change_dir, copy_dir, copy_file, remove_dir, which
from easybuild.tools.run import run_cmd
from easybuild.tools.systemtools import AARCH64, POWER, X86_64, get_cpu_architecture, get_shared_lib_ext
from easybuild.tools.utilities import nub


class EB_Java(PackedBinary):
    """Support for installing Java as a packed binary file (.tar.gz)
    Use the PackedBinary easyblock and set some extra paths.
    """

    def __init__(self, *args, **kwargs):
        """ Init the Java easyblock adding a new jdkarch template var """
        myarch = get_cpu_architecture()
        if myarch == AARCH64:
            jdkarch = 'aarch64'
        elif myarch == POWER:
            jdkarch = 'ppc64le'
        elif myarch == X86_64:
            jdkarch = 'x64'
        else:
            raise EasyBuildError("Architecture %s is not supported for Java on EasyBuild", myarch)

        super(EB_Java, self).__init__(*args, **kwargs)

        self.cfg.template_values['jdkarch'] = jdkarch
        self.cfg.generate_template_values()

    def extract_step(self):
        """Unpack the source"""
        if LooseVersion(self.version) < LooseVersion('1.7'):

            copy_file(self.src[0]['path'], self.builddir)
            adjust_permissions(os.path.join(self.builddir, self.src[0]['name']), stat.S_IXUSR, add=True)

            change_dir(self.builddir)
            run_cmd(os.path.join(self.builddir, self.src[0]['name']), log_all=True, simple=True, inp='')
        else:
            PackedBinary.extract_step(self)
            adjust_permissions(self.builddir, stat.S_IWUSR, add=True, recursive=True)

    def install_step(self):
        """Custom install step: just copy unpacked installation files."""
        if LooseVersion(self.version) < LooseVersion('1.7'):
            remove_dir(self.installdir)
            copy_dir(os.path.join(self.builddir, 'jdk%s' % self.version), self.installdir)
        else:
            PackedBinary.install_step(self)

    def post_install_step(self):
        """
        Custom post-installation step:
        - ensure correct glibc is used when installing into custom sysroot and using RPATH
        """
        super(EB_Java, self).post_install_step()

        # patch binaries and libraries when using alternate sysroot in combination with RPATH
        sysroot = build_option('sysroot')
        if sysroot and self.toolchain.use_rpath:
            if not which('patchelf'):
                error_msg = "patchelf not found via $PATH, required to patch RPATH section in binaries/libraries"
                raise EasyBuildError(error_msg)

            try:
                # list of paths in sysroot to consider for adding to RPATH section
                sysroot_lib_paths = glob.glob(os.path.join(sysroot, 'lib*'))
                sysroot_lib_paths += glob.glob(os.path.join(sysroot, 'usr', 'lib*'))
                sysroot_lib_paths += glob.glob(os.path.join(sysroot, 'usr', 'lib*', 'gcc', '*', '*'))
                if sysroot_lib_paths:
                    self.log.info("List of library paths in %s to add to RPATH section: %s", sysroot, sysroot_lib_paths)

                # find path to ELF interpreter
                elf_interp = None

                for ld_glob_pattern in (r'ld-linux-*.so.*', r'ld*.so.*'):
                    res = glob.glob(os.path.join(sysroot, 'lib*', ld_glob_pattern))
                    self.log.debug("Paths for ELF interpreter via '%s' pattern: %s", ld_glob_pattern, res)
                    if res:
                        # if there are multiple hits, make sure they resolve to the same paths,
                        # but keep using the symbolic link, not the resolved path!
                        real_paths = nub([os.path.realpath(x) for x in res])
                        if len(real_paths) == 1:
                            elf_interp = res[0]
                            self.log.info("ELF interpreter found at %s", elf_interp)
                            break
                        else:
                            raise EasyBuildError("Multiple different unique ELF interpreters found: %s", real_paths)

                if elf_interp is None:
                    raise EasyBuildError("Failed to isolate ELF interpreter!")

                module_guesses = self.make_module_req_guess()

                bindirs = [os.path.join(self.installdir, bindir) for bindir in module_guesses['PATH'] if
                           os.path.exists(os.path.join(self.installdir, bindir))]
                # Make sure these are unique real paths
                bindirs = list(set([os.path.realpath(path) for path in bindirs]))
                for bindir in bindirs:
                    for path in os.listdir(bindir):
                        path = os.path.join(bindir, path)
                        out, _ = run_cmd("file %s" % path, trace=False)
                        if "dynamically linked" in out:

                            out, _ = run_cmd("patchelf --print-interpreter %s" % path, trace=False)
                            self.log.debug("ELF interpreter for %s: %s" % (path, out))

                            run_cmd("patchelf --set-interpreter %s %s" % (elf_interp, path), trace=False)

                            out, _ = run_cmd("patchelf --print-interpreter %s" % path, trace=False)
                            self.log.debug("ELF interpreter for %s: %s" % (path, out))

                            out, _ = run_cmd("patchelf --print-rpath %s" % path, simple=False, trace=False)
                            curr_rpath = out.strip()
                            self.log.debug("RPATH for %s: %s" % (path, curr_rpath))

                            new_rpath = ':'.join([curr_rpath] + sysroot_lib_paths)
                            # note: it's important to wrap the new RPATH value in single quotes,
                            # to avoid magic values like $ORIGIN being resolved by the shell
                            run_cmd("patchelf --set-rpath '%s' %s" % (new_rpath, path), trace=False)

                            curr_rpath, _ = run_cmd("patchelf --print-rpath %s" % path, simple=False, trace=False)
                            self.log.debug("RPATH for %s (prior to shrinking): %s" % (path, curr_rpath))

                            run_cmd("patchelf --shrink-rpath %s" % path, trace=False)

                            curr_rpath, _ = run_cmd("patchelf --print-rpath %s" % path, simple=False, trace=False)
                            self.log.debug("RPATH for %s (after shrinking): %s" % (path, curr_rpath))

                libdirs = [os.path.join(self.installdir, libdir) for libdir in module_guesses['LIBRARY_PATH'] if
                           os.path.exists(os.path.join(self.installdir, libdir))]
                # Make sure these are unique real paths
                libdirs = list(set([os.path.realpath(path) for path in libdirs]))
                shlib_ext = '.' + get_shared_lib_ext()
                for libdir in libdirs:
                    for path, _, filenames in os.walk(libdir):
                        shlibs = [os.path.join(path, x) for x in filenames if x.endswith(shlib_ext)]
                        for shlib in shlibs:
                            out, _ = run_cmd("patchelf --print-rpath %s" % shlib, simple=False, trace=False)
                            curr_rpath = out.strip()
                            self.log.debug("RPATH for %s: %s" % (shlib, curr_rpath))

                            new_rpath = ':'.join([curr_rpath] + sysroot_lib_paths)
                            # note: it's important to wrap the new RPATH value in single quotes,
                            # to avoid magic values like $ORIGIN being resolved by the shell
                            run_cmd("patchelf --set-rpath '%s' %s" % (new_rpath, shlib), trace=False)

                            curr_rpath, _ = run_cmd("patchelf --print-rpath %s" % shlib, simple=False, trace=False)
                            self.log.debug("RPATH for %s (prior to shrinking): %s" % (path, curr_rpath))

                            run_cmd("patchelf --shrink-rpath %s" % shlib, trace=False)

                            curr_rpath, _ = run_cmd("patchelf --print-rpath %s" % shlib, simple=False, trace=False)
                            self.log.debug("RPATH for %s (after shrinking): %s" % (path, curr_rpath))

            except OSError as err:
                raise EasyBuildError("Failed to patch RPATH section in binaries/libraries: %s", err)

    def sanity_check_step(self):
        """Custom sanity check for Java."""
        custom_paths = {
            'files': ['bin/java', 'bin/javac'],
            'dirs': ['lib'],
        }

        custom_commands = [
            "java -help",
            "javac -help",
        ]

        super(EB_Java, self).sanity_check_step(custom_paths=custom_paths, custom_commands=custom_commands)

    def make_module_extra(self):
        """
        Set $JAVA_HOME to installation directory
        """
        txt = PackedBinary.make_module_extra(self)
        txt += self.module_generator.set_environment('JAVA_HOME', self.installdir)
        return txt



##
# Copyright 2009-2023 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
"""
EasyBuild support for FreeFem++, implemented as an easyblock

@author: Balazs Hajgato (Free University Brussels - VUB)
@author: Kenneth Hoste (HPC-UGent)
"""
import os
import re

import easybuild.tools.environment as env
import easybuild.tools.toolchain as toolchain
from easybuild.easyblocks.generic.configuremake import ConfigureMake
from easybuild.tools.build_log import EasyBuildError
from easybuild.tools.config import ERROR
from easybuild.tools.modules import get_software_root
from easybuild.tools.filetools import change_dir
from easybuild.tools.run import check_log_for_errors, run_cmd


class EB_FreeFEM(ConfigureMake):
    """Support for building and installing FreeFem++."""

    def configure_step(self):
        """
        FreeFem++ configure should run twice.
        First to configure PETSc (and then build it),
        then to configure FreeFem++ with the built PETSc.
        """

        # first Autoreconf has to be run
        if not get_software_root('Autoconf'):
            raise EasyBuildError("Autoconf is required to build FreeFem++. Please add it as build dependency")

        run_cmd("autoreconf -i", log_all=True, simple=False)

        configopts = [
            '--disable-optim',  # disable custom optimizations (not needed, $CFLAGS set by EasyBuild is picked up)
            '--enable-download',  # enable downloading of dependencies
            '--disable-openblas',  # do not download OpenBLAS
        ]

        blas_family = self.toolchain.blas_family()
        if blas_family == toolchain.OPENBLAS:
            openblas_root = get_software_root('OpenBLAS')
            configopts.append('--with-blas=%s' % os.path.join(openblas_root, 'lib'))
        elif blas_family == toolchain.INTELMKL:
            mkl_root = os.getenv('MKLROOT')
            configopts.append("--with-mkl=%s" % os.path.join(mkl_root, 'mkl', 'lib', 'intel64'))

        # specify which MPI to build with based on MPI component of toolchain
        if self.toolchain.mpi_family() in toolchain.OPENMPI:
            self.cfg.update('configopts', '--with-mpi=openmpi')
        elif self.toolchain.mpi_family() in toolchain.INTELMPI:
            self.cfg.update('configopts', '--with-mpi=mpic++')

        # if usempi is enabled, configure with --with-mpi=yes
        elif self.toolchain.options.get('usempi', None):
            self.cfg.update('configopts', '--with-mpi=yes')
        else:
            self.cfg.update('configopts', '--with-mpi=no')

        # check dependencies and add the corresponding configure options for FreeFEM
        hdf5_root = get_software_root('HDF5')
        if hdf5_root:
            configopts.append('--with-hdf5=%s ' % os.path.join(hdf5_root, 'bin', 'h5pcc'))

        gsl_root = get_software_root('GSL')
        if gsl_root:
            configopts.append('--with-gsl-prefix=%s' % gsl_root)

        petsc_root = get_software_root('PETSc')
        if petsc_root:
            configopts.append('--with-petsc=%s' % os.path.join(petsc_root, 'lib', 'petsc', 'conf', 'petscvariables'))

        bemtool_root = get_software_root('BemTool')
        if bemtool_root:
            configopts.append('--with-bem-include=%s' % bemtool_root)

        for configopt in configopts:
            self.cfg.update('configopts', configopt)

        # initial configuration
        out = super(EB_FreeFEM, self).configure_step()

        regex = re.compile("WARNING: unrecognized options: (.*)", re.M)
        res = regex.search(out)
        if res:
            raise EasyBuildError("One or more configure options not recognized: %s" % res.group(1))

        if not petsc_root:
            # re-create installation dir (deletes old installation),
            # then set keeppreviousinstall to True (to avoid deleting PETSc installation)
            self.make_installdir()
            self.cfg['keeppreviousinstall'] = True

            # configure and make petsc-slepc
            # download & build PETSc as recommended by FreeFEM if no PETSc dependency was provided
            cwd = change_dir(os.path.join('3rdparty', 'ff-petsc'))

            cmd = ['make']
            if self.cfg['parallel']:
                cmd.append('-j %s' % self.cfg['parallel'])
            cmd.append('petsc-slepc')

            run_cmd(' '.join(cmd), log_all=True, simple=False)

            change_dir(cwd)

            # reconfigure for FreeFEM build
            super(EB_FreeFEM, self).configure_step()

    def test_step(self):
        """Run tests."""

        # run tests unless they're disabled explicitly
        if self.cfg['runtest'] is None or self.cfg['runtest']:

            # avoid oversubscribing, by using both OpenMP threads and having OpenBLAS use threads as well
            env.setvar('OMP_NUM_THREADS', '1')

            (out, _) = run_cmd("make check", log_all=True, simple=False, regexp=False)

            # Raise an error if any test failed
            check_log_for_errors(out, [('# (ERROR|FAIL): +[1-9]', ERROR)])

    def sanity_check_step(self):
        """Custom sanity check step for FreeFem++."""

        binaries = [os.path.join('bin', x) for x in ['bamg', 'cvmsh2', 'ffglut', 'ffmaster', 'ffmedit']]
        binaries.extend(os.path.join('bin', 'ff-%s' % x) for x in ['c++', 'get-dep', 'mpirun', 'pkg-download'])
        binaries.extend(os.path.join('bin', 'FreeFem++%s' % x) for x in ['', '-mpi', '-nw'])

        custom_paths = {
            'files': binaries,
            'dirs': [os.path.join('share', 'FreeFEM', self.version)] +
                    [os.path.join('lib', 'ff++', self.version, x) for x in ['bin', 'etc', 'idp', 'include', 'lib']],
        }
        super(EB_FreeFEM, self).sanity_check_step(custom_paths=custom_paths)

    def make_module_extra(self):
        txt = super(EB_FreeFEM, self).make_module_extra()
        if self.toolchain.blas_family() == toolchain.OPENBLAS:
            # avoid oversubscribing the available cores, by only running one OpenMP thread
            txt += self.module_generator.set_environment('OMP_NUM_THREADS', "1")
        return txt
